@@ -2,12 +2,12 @@
 /*****************************************************************************
  * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
  *
- * $Id: config.c,v 4.9 1997/08/03 13:29:45 mj Exp $
+ * $Id: config.c,v 4.22 1999/05/15 20:54:40 mj Exp $
  *
  * Configuration data and functions
  *
  *****************************************************************************
- * Copyright (C) 1990-1997
+ * Copyright (C) 1990-1999
  *  _____ _____
  * |     |___  |   Martin Junius             FIDO:      2:2452/110
  * | | | |   | |   Radiumstr. 18             Internet:  mj@fido.de
@@ -34,20 +34,6 @@
 
 
 
-/*
- * Current line in config file
- */
-static int scf_line;
-
-
-
-/*
- * Directories
- */
-static char scf_libdir[MAXPATH];
-static char scf_spooldir[MAXPATH];
-static char scf_logdir[MAXPATH];
-
 /*
  * Hostname / domainname
  */
@@ -65,6 +51,7 @@ struct st_addr
     int zone;				/* Zone for this address set */
     Node addr;				/* Our own main address */
     Node uplink;			/* Uplink address */
+    Node gateaddr;			/* For new-style GateAddress config */
 };
 
 static struct st_addr scf_addr[MAXADDRESS];
@@ -77,6 +64,7 @@ static Node scf_c_uplink = { -1, -1, -1, -1, "" };	/* Current uplink  */
 
 static int scf_ia = 0;			/* Index for Address */
 static int scf_ir = 0;			/* Index for Uplink */
+static int scf_ig = 0;			/* Index for GateAddress */
 
 
 
@@ -130,23 +118,77 @@ static struct st_cflist *scf_list_last  = NULL;
 
 
 
+/*
+ * Set config for gateway programs (ftn2rfc, rfc2ftn, ftnin, ftnmail),
+ * basically reshuffling addresses a bit (using GateAddress as the main
+ * address).
+ */
+void cf_i_am_a_gateway_prog(void)
+{
+    int i;
+    
+    if(scf_ig) 
+    {
+	/* GateAddress used, new-style config, shuffle addresses */
+	debug(8, "config: switching to gateway, using GateAddress");
+	for(i=0; i<scf_ig; i++)
+	{
+	    scf_addr[i].uplink = scf_addr[i].addr;
+	    scf_addr[i].addr   = scf_addr[i].gateaddr;
+	}
+	for(i=0; i<scf_ig; i++)
+	    debug(8, "config: address Z%-4d: GATE addr=%s uplink=%s",
+		  scf_addr[i].zone, znfp1(&scf_addr[i].addr),
+		  znfp2(&scf_addr[i].uplink)                             );
+	return;
+    }
+
+    if(scf_ir == 0)
+    {
+	/* No Uplink used, same address for gateway and tosser, copy addr */
+	debug(8, "config: no explicit uplink, using Address");
+	for(i=0; i<scf_ia; i++)
+	    scf_addr[i].uplink = scf_addr[i].addr;
+	scf_ir = scf_ia;
+	for(i=0; i<scf_ia; i++)
+	    debug(8, "config: address Z%-4d: GATE addr=%s uplink=%s",
+		  scf_addr[i].zone, znfp1(&scf_addr[i].addr),
+		  znfp2(&scf_addr[i].uplink)                             );
+	return;
+    }
+}
+
+
+
 /*
  * Check for corresponding "Address" and "Uplink" entries (config.gate)
  */
 void cf_check_gate(void)
 {
-    if(scf_ir == 0)
+    if(scf_ia == 0)
     {
-	log("ERROR: config: no uplink");
+	log("ERROR: config: no Address");
 	if(!verbose)
-	    fprintf(stderr, "ERROR: config: no uplink");
+	    fprintf(stderr, "ERROR: config: no Address\n");
 	exit(EX_USAGE);
     }
-    
-    if(scf_ia != scf_ir)
+
+#if 0    
+    if(scf_ir==0 && scf_ig==0)
     {
-	log("WARNING: config: %d address != %d uplink");
+	log("ERROR: config: no Uplink or GateAddress");
+	if(!verbose)
+	    fprintf(stderr, "ERROR: config: no Uplink or GateAddress\n");
+	exit(EX_USAGE);
     }
+#endif
+    
+    if(scf_ir && scf_ia!=scf_ir)
+	log("WARNING: config: #Address (%d) != #Uplink (%d)",
+	    scf_ia, scf_ir);
+    if(scf_ig && scf_ia!=scf_ig)
+	log("WARNING: config: #Address (%d) != #GateAddress (%d)",
+	    scf_ia, scf_ig);
 }
 
 
@@ -158,22 +200,19 @@ void cf_debug(void)
 {
     int i;
 
-    debug(8, "config: libdir=%s  spooldir=%s  logdir=%s",
-	  scf_libdir, scf_spooldir, scf_logdir);
     debug(8, "config: fqdn=%s", scf_fqdn);
     
     for(i=0; i<scf_naddr; i++)
-	debug(8, "config: address Z%-4d: %s  Uplink: %s",
-	      scf_addr[i].zone,
-	      node_to_asc(&scf_addr[i].addr, TRUE),
-	      node_to_asc(&scf_addr[i].uplink, TRUE)          );
+	debug(8, "config: address Z%-4d: addr=%s uplink=%s gateaddr=%s",
+	      scf_addr[i].zone, znfp1(&scf_addr[i].addr),
+	      znfp2(&scf_addr[i].uplink), znfp3(&scf_addr[i].gateaddr));
 
     for(i=0; i<scf_nzones; i++)
 	debug(8, "config: zone %-4d: %s  %s  %s", scf_zones[i].zone,
 	      scf_zones[i].inet_domain, scf_zones[i].ftn_domain,
 	      scf_zones[i].out                                 );
 
-    debug(8, "config: gateway=%s", node_to_asc(&scf_gateway, TRUE));
+    debug(8, "config: gateway=%s", znfp1(&scf_gateway));
 }
 
 
@@ -240,8 +279,8 @@ void cf_set_zone(int zone)
 	    scf_c_uplink = scf_addr[i].uplink;
 	    debug(9, "Select Z%d AKA: %s  Uplink: %s",
 		  scf_addr[i].zone,
-		  node_to_asc(&scf_addr[i].addr, TRUE),
-		  node_to_asc(&scf_addr[i].uplink, TRUE)          );
+		  znfp1(&scf_addr[i].addr),
+		  znfp2(&scf_addr[i].uplink)          );
 	    return;
 	}
     
@@ -249,8 +288,8 @@ void cf_set_zone(int zone)
     scf_c_addr   = scf_addr[i].addr;
     scf_c_uplink = scf_addr[i].uplink;
     debug(9, "Select default AKA: %s  Uplink: %s",
-	  node_to_asc(&scf_addr[i].addr, TRUE),
-	  node_to_asc(&scf_addr[i].uplink, TRUE)          );
+	  znfp1(&scf_addr[i].addr),
+	  znfp2(&scf_addr[i].uplink)          );
 }
 
 
@@ -285,18 +324,32 @@ int cf_defzone(void)
  * comments (starting with `#'), and empty lines. cf_getline() returns
  * a pointer to the first non-whitespace in buffer.
  */
+static long cf_lineno = 0;
+
+long cf_lineno_get(void)
+{
+    return cf_lineno;
+}
+
+long cf_lineno_set(long n)
+{
+    long old;
+    
+    old = cf_lineno;
+    cf_lineno = n;
+
+    return old;
+}
+
 char *cf_getline(char *buffer, int len, FILE *fp)
 {
     char *p;
 
-    scf_line = 0;
     while (fgets(buffer, len, fp)) {
-	scf_line++;
+	cf_lineno++;
 	strip_crlf(buffer);
-	if((p = strchr(buffer, '#')))		/* Strip comments */
-	    *p = 0;
-	for(p=buffer; *p && is_space(*p); p++) ;	/* Skip white spaces */
-	if (*p)
+	for(p=buffer; *p && is_space(*p); p++) ; /* Skip white spaces */
+	if (*p != '#')
 	    return p;
     }
     return NULL;
@@ -310,7 +363,8 @@ char *cf_getline(char *buffer, int len, FILE *fp)
 void cf_do_line(char *line)
 {
     char *p, *keyword;
-
+    Node a;
+	    
     keyword = xstrtok(line, " \t");
     if(!keyword)
 	return;
@@ -373,44 +427,9 @@ void cf_do_line(char *line)
 	else
 	    BUF_COPY(scf_hostsdomain, p);
     }
-    /***** libdir *******************************************************/
-    else if (!stricmp(keyword, "libdir"))
-    {
-	p = xstrtok(NULL, " \t");
-	if(!p) 
-	{
-	    log("config: missing libdir");
-	    return;
-	}
-	BUF_COPY(scf_libdir, p);
-    }
-    /***** spooldir *****************************************************/
-    else if (!stricmp(keyword, "spooldir"))
-    {
-	p = xstrtok(NULL, " \t");
-	if(!p) 
-	{
-	    log("config: missing spooldir");
-	    return;
-	}
-	BUF_COPY(scf_spooldir, p);
-    }
-    /***** logdir *******************************************************/
-    else if (!stricmp(keyword, "logdir"))
-    {
-	p = xstrtok(NULL, " \t");
-	if(!p) 
-	{
-	    log("config: missing logdir");
-	    return;
-	}
-	BUF_COPY(scf_logdir, p);
-    }
     /***** address ******************************************************/
     else if (!stricmp(keyword, "address" ))
     {
-	Node a;
-	    
 	/* address */
 	p = xstrtok(NULL, " \t");
 	if(!p) 
@@ -436,9 +455,6 @@ void cf_do_line(char *line)
     /***** uplink *******************************************************/
     else if (!stricmp(keyword, "uplink" ))
     {
-	Node a;
-	    
-	/* Main address */
 	p = xstrtok(NULL, " \t");
 	if(!p) 
 	{
@@ -459,6 +475,29 @@ void cf_do_line(char *line)
 	else
 	    log("config: too many addresses");
     }
+    /***** GateAddress **************************************************/
+    else if (!stricmp(keyword, "gateaddress" ))
+    {
+	p = xstrtok(NULL, " \t");
+	if(!p) 
+	{
+	    log("config: missing address");
+	    return;
+	}
+	if( asc_to_node(p, &a, FALSE) == ERROR )
+	{
+	    log("config: illegal address %s", p);
+	    return;
+	}
+
+	if(scf_ig < MAXADDRESS)
+	{
+	    scf_addr[scf_ig].gateaddr = a;
+	    scf_ig++;
+	}
+	else
+	    log("config: too many addresses");
+    }
     /***** zone *********************************************************/
     else if (!stricmp(keyword, "zone" ))
     {
@@ -540,9 +579,6 @@ void cf_do_line(char *line)
     /***** gateway ******************************************************/
     else if (!stricmp(keyword, "gateway" ))
     {
-	Node a;
-	    
-	/* Main address */
 	p = xstrtok(NULL, " \t");
 	if(!p) 
 	{
@@ -619,20 +655,24 @@ void cf_initialize(void)
 {
     char *p;
 
-    if( (p = getenv("FIDOGATE")) )
-	BUF_COPY(scf_libdir, p);
-    else
-	BUF_COPY(scf_libdir, LIBDIR);
-    BUF_COPY(scf_spooldir, SPOOLDIR);
-    BUF_COPY(scf_logdir, LOGDIR);
-
-    /*
-     * Check for real uid != effective uid, setuid installed FIDOGATE
-     * programs, and disable debug() output (-v on command line) in
-     * this case.
+    /* 
+     * Check for real uid != effective uid (setuid installed FIDOGATE
+     * programs), disable debug() output (-v on command line) and
+     * environment variables in this case.
      */
     if(getuid() != geteuid())
+    {
 	no_debug = TRUE;
+	return;
+    }
+    
+    /* Env FIDOGATE is alias for FIDOGATE_CONFIGDIR */
+    if( (p = getenv("FIDOGATE")) )
+	cf_s_configdir(p);
+
+    /* Code for reading FIDOGATE_xxx env vars generated by subst.pl */
+#   include "cf_env.c"
+
 }
 
 
@@ -655,6 +695,9 @@ void cf_set_addr(char *addr)
     }
 
     scf_naddr        = 1;
+    scf_ia           = 1;
+    scf_ig           = 0;
+    scf_ir           = 0;
     scf_addr[0].zone = node.zone;
     scf_addr[0].addr = node;
     scf_zone         = node.zone;
@@ -684,6 +727,7 @@ void cf_set_uplink(char *addr)
     }
     
     scf_naddr          = 1;
+    scf_ir             = 1;
     scf_addr[0].uplink = node;
     scf_zone           = scf_addr[0].zone;
     scf_index          = 0;
@@ -693,41 +737,6 @@ void cf_set_uplink(char *addr)
 
 
 
-/*
- * Set/get lib/spool/log directory
- */
-void cf_set_libdir(char *dir)
-{
-    BUF_COPY(scf_libdir, dir);
-}
-
-void cf_set_spooldir(char *dir)
-{
-    BUF_COPY(scf_spooldir, dir);
-}
-
-void cf_set_logdir(char *dir)
-{
-    BUF_COPY(scf_logdir, dir);
-}
-
-char *cf_libdir(void)
-{
-    return scf_libdir;
-}
-
-char *cf_spooldir(void)
-{
-    return scf_spooldir;
-}
-
-char *cf_logdir(void)
-{
-    return scf_logdir;
-}
-
-
-
 /*
  * Return hostname / domain name / fully qualified domain name
  */
@@ -985,199 +994,6 @@ char *cf_get_string(char *name, int first)
 
 
 
-/*
- * Get config values for Aliases, Areas, Hosts, Passwd, Packing, Routing
- * config files.
- */
-char *cf_p_aliases(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Aliases", TRUE)) )
-	    pval = DEF_ALIASES;
-	debug(8, "config: Aliases %s", pval);
-    }
-
-    return pval;
-}
-
-char *cf_p_areas(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Areas", TRUE)) )
-	    pval = DEF_AREAS;
-	debug(8, "config: Areas %s", pval);
-    }
-
-    return pval;
-}
-
-char *cf_p_hosts(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Hosts", TRUE)) )
-	    pval = DEF_HOSTS;
-	debug(8, "config: Hosts %s", pval);
-    }
-
-    return pval;
-}
-
-char *cf_p_passwd(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Passwd", TRUE)) )
-	    pval = DEF_PASSWD;
-	debug(8, "config: Passwd %s", pval);
-    }
-
-    return pval;
-}
-
-char *cf_p_packing(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Packing", TRUE)) )
-	    pval = DEF_PACKING;
-	debug(8, "config: Packing %s", pval);
-    }
-
-    return pval;
-}
-
-char *cf_p_routing(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Routing", TRUE)) )
-	    pval = DEF_ROUTING;
-	debug(8, "config: Routing %s", pval);
-    }
-
-    return pval;
-}
-
-char *cf_p_history(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("History", TRUE)) )
-	    pval = DEF_HISTORY;
-	debug(8, "config: History %s", pval);
-    }
-
-    return pval;
-}
-
-
-
-/*
- * Get config values for Inbound, PInbound, UUInbound, Outbound
- */
-char *cf_p_inbound(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Inbound", TRUE)) )
-	    pval = INBOUND;
-	debug(8, "config: Inbound %s", pval);
-    }
-
-    return pval;
-}
-
-static char *cf_p_s_pinbound(char *s)
-{
-    static char *pval;
-
-    if(s)
-	pval = strsave(s);
-    
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("PInbound", TRUE)) )
-	    if( ! (pval = cf_get_string("Inbound", TRUE)) )
-		pval = PINBOUND;
-	debug(8, "config: PInbound %s", pval);
-    }
-
-    return pval;
-}
-    
-char *cf_p_pinbound(void)
-{
-    return cf_p_s_pinbound(NULL);
-}
-
-char *cf_s_pinbound(char *s)
-{
-    return cf_p_s_pinbound(s);
-}
-
-char *cf_p_uuinbound(void)
-{
-    static char *pval;
-
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("UUInbound", TRUE)) )
-	    if( ! (pval = cf_get_string("Inbound", TRUE)) )
-		pval = INBOUND;
-	debug(8, "config: UUInbound %s", pval);
-    }
-
-    return pval;
-}
-
-static char *cf_p_s_outbound(char *s)
-{
-    static char *pval;
-
-    if(s)
-	pval = strsave(s);
-    
-    if(! pval)
-    {
-	if( ! (pval = cf_get_string("Outbound", TRUE)) )
-	    pval = OUTBOUND;
-	debug(8, "config: Outbound %s", pval);
-    }
-
-    return pval;
-}
-    
-char *cf_p_outbound(void)
-{
-    return cf_p_s_outbound(NULL);
-}
-
-char *cf_s_outbound(char *s)
-{
-    return cf_p_s_outbound(s);
-}
-
-
-
 /*
  * Get config values for Organization, Origin
  */
