@@ -2,12 +2,12 @@
 /*****************************************************************************
  * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
  *
- * $Id: message.c,v 4.8 1997/07/25 21:01:39 mj Exp $
+ * $Id: message.c,v 4.16 1999/10/04 08:18:21 mj Exp $
  *
  * Reading and processing FTN text body
  *
  *****************************************************************************
- * Copyright (C) 1990-1997
+ * Copyright (C) 1990-1999
  *  _____ _____
  * |     |___  |   Martin Junius             FIDO:      2:2452/110
  * | | | |   | |   Radiumstr. 18             Internet:  mj@fido.de
@@ -42,7 +42,7 @@ static void debug_line(FILE *out, char *line, int crlf)
     int c;
     
     while( (c = *line++) )
-	if( !(c & 0x60) )
+	if( !(c & 0xe0) )
 	{
 	    if(crlf || (c!='\r' && c!='\n'))
 		fprintf(out, "^%c", '@' + c);
@@ -188,7 +188,7 @@ int pkt_get_body(FILE *fp, Textlist *tl)
     if( (type==MSG_END || type==MSG_TYPE) && buffer[0] )
     {
 	/* Make sure that this line is terminated by \r\n */
-	strncat0(buffer, "\r\n", sizeof(buffer));
+	BUF_APPEND(buffer, "\r\n");
 
 	tl_append(tl, buffer);
     }
@@ -262,7 +262,10 @@ static int msg_body_parse_echomail(MsgBody *body)
 	p && strncmp(p->line, "\001PATH", 5) && strncmp(p->line, "SEEN-BY", 7);
 	p=p->prev) ;
     if(p == NULL)
+    {
+	log("ERROR: parsing echomail message: no ^APATH or SEEN-BY line");
 	return -2;
+    }
     /* ^APATH */
     for(; p && !strncmp(p->line, "\001PATH", 5); p=p->prev) ;
     /* SEEN-BY */
@@ -341,7 +344,10 @@ static int msg_body_parse_echomail(MsgBody *body)
     }
 
     if(body->seenby.n==0 /*|| body->path.n==0*/)
+    {
+	log("ERROR: parsing echomail message: no SEEN-BY line");
 	return -2;
+    }
     if(body->tear==NULL || body->origin==NULL /** || body->path.n==0 **/)
 	return -1;
     
@@ -420,7 +426,7 @@ static int msg_body_parse_netmail(MsgBody *body)
 }
 
 
-static int is_blank_line(char *s)
+int is_blank_line(char *s)
 {
     if(!s)
 	return TRUE;
@@ -602,21 +608,12 @@ int msg_put_msgbody(FILE *fp, MsgBody *body, int term)
 
 
 /*
- * Convert text line read from FTN message body, using charset_xlate()
+ * Convert text line read from FTN message body
  */
-char *msg_xlate_line(char *buf, int n, char *line, int cvt8)
-              				/* Buffer for converted text */
-          				/* Size of buffer */
-               				/* Input */
+char *msg_xlate_line(char *buf, int n, char *line, int qp)
 {
     char *s, *p, *xl;
-    int c, cidx;
-
-    cidx = 0;
-    if(cvt8 & AREA_8BIT)
-	cidx = 1;
-    if(cvt8 & AREA_QP)
-	cidx = 2;
+    int c;
 
     n--;				/* Room for \0 char */
 
@@ -641,7 +638,7 @@ char *msg_xlate_line(char *buf, int n, char *line, int cvt8)
 		c = c + '@';
 	    }
 	}
-	else if(cidx==2 && c=='=')
+	else if(qp && c=='=')
 	{
 	    /* Translate '=' to MIME quoted-printable =3D */
 	    xl = "=3D";
@@ -656,7 +653,7 @@ char *msg_xlate_line(char *buf, int n, char *line, int cvt8)
 	else if(c & 0x80)
 	{
 	    /* Translate special characters according to character set */
-	    xl = charset_xlate(c, cidx);
+	    xl = charset_map_c(c, qp);
 	    if(!xl || !*xl)
 		continue;
 	    while(*xl)
@@ -687,40 +684,64 @@ char *msg_xlate_line(char *buf, int n, char *line, int cvt8)
  * Format buffer line and put it into Textlist. Returns number of
  * lines.
  */
+#define DEFAULT_LINE_LENGTH	72
+#define MAX_LINE_LENGTH		200
+
+static int msg_get_line_length(void)
+{
+    static int message_line_length = 0;
+
+    if(!message_line_length) 
+    {
+	char *p;
+	if( (p = cf_get_string("MessageLineLength", TRUE)) )
+	{
+	    debug(8, "config: MessageLineLength %s", p);
+	    message_line_length = atoi(p);
+	    if(message_line_length < 20 ||
+	       message_line_length > MAX_LINE_LENGTH) 
+	    {
+		log("WARNING: illegal MessageLineLength value %d",
+		    message_line_length);
+		message_line_length = DEFAULT_LINE_LENGTH;
+	    }
+	}
+	else
+	    message_line_length = DEFAULT_LINE_LENGTH;
+    }
+    return message_line_length;
+}
+
+
 int msg_format_buffer(char *buffer, Textlist *tlist)
 {
-    /*
-     * Remark: MAX_LINELEN+16 and MAX_LINELEN+8, the numbers +16 and +8
-     *         are just arbitrary values to be safe.
-     */
+    int max_linelen;
     char *p, *np;
-    char localbuffer[MAX_LINELEN + 16];
+    char localbuffer[MAX_LINE_LENGTH + 16];	/* Some extra space */
     int i;
     int lines;
+
+    max_linelen = msg_get_line_length();
     
-    if(strlen(buffer) <= MAX_LINELEN)		/* Nothing to do */
+    if(strlen(buffer) <= max_linelen)		/* Nothing to do */
     {
 	tl_append(tlist, buffer);
 	return 1;
     }
     else
     {
-	/*
-	 * Break line with word wrap
-	 */
+	/* Break line with word wrap */
 	lines = 0;
 	p     = buffer;
 
 	while(TRUE)
 	{
-	    /*
-	     * Search backward for a whitespace to break line. If no proper
-	     * point is found, the line will not be splitted.
-	     */
-	    for(i=MAX_LINELEN-1; i>=0; i--)
+	    /* Search backward for a whitespace to break line. If no
+	     * proper point is found, the line will not be split.  */
+	    for(i=max_linelen-1; i>=0; i--)
 		if(is_blank(p[i]))	/* Found a white space */
 		    break;
-	    if(i < MAX_LINELEN/2)	/* Not proper space to split found, */
+	    if(i < max_linelen/2)	/* Not proper space to split found, */
 	    {				/* put line as is                   */
 		tl_append(tlist, p);
 		lines++;
@@ -729,24 +750,20 @@ int msg_format_buffer(char *buffer, Textlist *tlist)
 	    for(; i>=0 && is_blank(p[i]); i--);	/* Skip more white space */
 	    i++;				/* Return to last white sp. */
 
-	    /*
-	     * Cut here and put into textlist
-	     */
+	    /* Cut here and put into textlist */
 	    np = p + i;
 	    *np++ = 0;
-	    strncpy0(localbuffer, p, MAX_LINELEN+8);
-	    strcat(localbuffer, "\n");
+	    BUF_COPY2(localbuffer, p, "\n");
 	    tl_append(tlist, localbuffer);
 	    lines++;
 	    
-	    /*
-	     * Advance buffer pointer and test length of remaining line
-	     */
+	    /* Advance buffer pointer and test length of remaining
+	     * line */
 	    p = np;
 	    for(; *p && is_blank(*p); p++);	/* Skip white space */
 	    if(*p == 0)				/* The end */
 		return lines;
-	    if(strlen(p) <= MAX_LINELEN)	/* No more wrappin' */
+	    if(strlen(p) <= max_linelen)	/* No more wrappin' */
 	    {
 		tl_append(tlist, p);
 		lines++;
@@ -792,9 +809,7 @@ int msg_parse_origin(char *buffer, Node *node)
     *right = 0;
     *left++ = 0;
 
-    /*
-     * Parse node info
-     */
+    /* Parse node info */
     while(*left && !is_digit(*left))
 	left++;
     if(asc_to_node(left, node, FALSE) != OK)
