@@ -2,12 +2,12 @@
 /*****************************************************************************
  * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
  *
- * $Id: packet.c,v 4.3 1996/12/17 17:19:46 mj Exp $
+ * $Id: packet.c,v 4.14 1999/05/22 12:05:00 mj Exp $
  *
  * Functions to read/write packets and messages
  *
  *****************************************************************************
- * Copyright (C) 1990-1997
+ * Copyright (C) 1990-1999
  *  _____ _____
  * |     |___  |   Martin Junius             FIDO:      2:2452/110
  * | | | |   | |   Radiumstr. 18             Internet:  mj@fido.de
@@ -55,7 +55,7 @@ static FILE *pkt_create		(Node *);
  */
 void pkt_outdir(char *dir1, char *dir2)
 {
-    str_expand_name(packet_dir, sizeof(packet_dir), dir1);
+    BUF_EXPAND(packet_dir, dir1);
     if(dir2)
     {
 	BUF_APPEND(packet_dir, "/" );
@@ -78,7 +78,7 @@ char *pkt_get_outdir(void)
  */
 void pkt_baddir(char *dir1, char *dir2)
 {
-    str_expand_name(packet_bad, sizeof(packet_bad), dir1);
+    BUF_EXPAND(packet_bad, dir1);
     if(dir2)
     {
 	BUF_APPEND(packet_bad, "/" );
@@ -128,14 +128,16 @@ static char *pkt_newname(char *name)
 {
     if(name)
     {
-	strncpy0(packet_name, name, sizeof(packet_name));
-	strncpy0(packet_tmp , name, sizeof(packet_tmp ));
+	BUF_COPY(packet_name, name);
+	BUF_COPY(packet_tmp , name);
     }
     else
     {
-	long n = sequencer(SEQ_PKT);
-	sprintf(packet_name, "%s/%08ld.pkt", packet_dir, n);
-	sprintf(packet_tmp , "%s/%08ld.tmp", packet_dir, n);
+	long n = sequencer(DEFAULT_SEQ_PKT);
+	str_printf(packet_name, sizeof(packet_name),
+		   "%s/%08ld.pkt", packet_dir, n);
+	str_printf(packet_tmp , sizeof(packet_tmp),
+		   "%s/%08ld.tmp", packet_dir, n);
     }
 
     return packet_name;
@@ -277,7 +279,7 @@ static FILE *pkt_open_node(Node *node, char *flav, int bsy)
 	pkt.time = time(NULL);
 	/* Password */
 	pwd = passwd_lookup("packet", node);
-	strncpy0(pkt.passwd, pwd ? pwd->passwd : "", sizeof(pkt.passwd));
+	BUF_COPY(pkt.passwd, pwd ? pwd->passwd : "");
 
 	/* Rest is filled in by pkt_put_hdr() */
 	if(pkt_put_hdr(fp, &pkt) == ERROR)
@@ -363,7 +365,7 @@ static FILE *pkt_create(Node *to)
     pkt.time = time(NULL);
     /* Password */
     pwd = passwd_lookup("packet", to);
-    strncpy0(pkt.passwd, pwd ? pwd->passwd : "", sizeof(pkt.passwd));
+    BUF_COPY(pkt.passwd, pwd ? pwd->passwd : "");
 
     /* Rest is filled in by pkt_put_hdr() */
     if(pkt_put_hdr(packet_file, &pkt) == ERROR)
@@ -524,9 +526,9 @@ void pkt_debug_msg_hdr(FILE *out, Message *msg, char *txt)
 {
     fprintf(out, "%sFTN message header:\n", txt);
     fprintf(out, "    From: %-36s @ %s\n",
-	    msg->name_from, node_to_asc(&msg->node_from, TRUE));
+	    msg->name_from, znfp1(&msg->node_from));
     fprintf(out, "    To  : %-36s @ %s\n",
-	    msg->name_to  , node_to_asc(&msg->node_to  , TRUE));
+	    msg->name_to  , znfp1(&msg->node_to));
     fprintf(out, "    Subj: %s\n", msg->subject);
     fprintf(out, "    Date: %s\n",
 	    msg->date!=-1 ? date(NULL, &msg->date) : "LOCAL" );
@@ -598,7 +600,7 @@ int pkt_put_date(FILE *pkt, time_t t)
     }
     
     /* Date according to FTS-0001 */
-    pkt_put_string(pkt, date("%d %b %y  %H:%M:%S", &t) );
+    pkt_put_string(pkt, date(DATE_FTS_0001, &t) );
 
     return ferror(pkt);
 }
@@ -608,6 +610,20 @@ int pkt_put_date(FILE *pkt, time_t t)
 /*
  * Write message header to packet.
  */
+static int force_fmpt0 = FALSE;
+static int force_intl  = TRUE;
+
+void pkt_set_forcefmpt0(int f)
+{
+    force_fmpt0 = f;
+}
+
+void pkt_set_forceintl(int f)
+{
+    force_intl = f;
+}
+
+
 int pkt_put_msg_hdr(FILE *pkt, Message *msg, int kludge_flag)
     /* kludge_flag --- TRUE: write AREA/^AINTL,^AFMPT,^ATOPT */
 {
@@ -640,26 +656,20 @@ int pkt_put_msg_hdr(FILE *pkt, Message *msg, int kludge_flag)
 	fprintf(pkt, "AREA:%s\r\n", msg->area);
     else 
     {
-#ifdef FTN_FORCE_INTL
-	if(TRUE)
-#else
-	if(cf_zone()    != msg->node_from.zone  ||
+	if(force_intl                           ||
+	   cf_zone()    != msg->node_from.zone  ||
 	   cf_defzone() != msg->node_from.zone  ||
 	   cf_zone()    != msg->node_to  .zone  ||
 	   cf_defzone() != msg->node_to  .zone     )
-#endif
 	{
 	    Node tmpf, tmpt;
 	    
 	    tmpf = msg->node_from; tmpf.point = 0; tmpf.domain[0] = 0;
 	    tmpt = msg->node_to;   tmpt.point = 0; tmpt.domain[0] = 0;
-	    fprintf(pkt, "\001INTL %s %s\r\n",
-		    node_to_asc(&tmpt, FALSE), node_to_asc(&tmpf, FALSE));
+	    fprintf(pkt, "\001INTL %s %s\r\n", znf1(&tmpt), znf2(&tmpf));
 	}
 
-/* Kludge: always generate ^AFMPT, because SQUISH adds point address,
- * if the origin address of the packet is a point address.
- **	if(msg->node_from.point)  **/
+	if(force_fmpt0 || msg->node_from.point)
 	    fprintf(pkt, "\001FMPT %d\r\n", msg->node_from.point);
 	if(msg->node_to  .point)
 	    fprintf(pkt, "\001TOPT %d\r\n", msg->node_to  .point);
@@ -733,6 +743,10 @@ int pkt_get_hdr(FILE *fp, Packet *pkt)
     pkt->passwd[0] = 0;
     pkt->capword   = 0;
 
+    /* Set zone to default, i.e. use the zone from your FIRST aka
+     * specified in fidogate.conf */
+    pkt->from.zone = pkt->to.zone = cf_defzone();
+
     /* Orig node */
     if((val = pkt_get_int16(fp)) == ERROR)  return ERROR;
     pkt->from.node = val;
@@ -817,7 +831,7 @@ int pkt_get_hdr(FILE *fp, Packet *pkt)
     pkt->capword = cw;
     /* Orig zone (FSC-0039) */
     if((val = pkt_get_int16(fp)) == ERROR)  return ERROR;
-    if(cw)
+    if(cw && val)
     {
 	pkt->from.zone = val;
 	if(ozone != val)
@@ -826,7 +840,7 @@ int pkt_get_hdr(FILE *fp, Packet *pkt)
     }
     /* Dest zone (FSC-0039) */
     if((val = pkt_get_int16(fp)) == ERROR)  return ERROR;
-    if(cw)
+    if(cw && val)
     {
 	pkt->to.zone = val;
 	if(dzone != val)
@@ -858,8 +872,8 @@ int pkt_get_hdr(FILE *fp, Packet *pkt)
 void pkt_debug_hdr(FILE *out, Packet *pkt, char *txt)
 {
     fprintf(out, "%sFTN packet header:\n", txt);
-    fprintf(out, "    From: %s\n", node_to_asc(&pkt->from, TRUE));
-    fprintf(out, "    To  : %s\n", node_to_asc(&pkt->to  , TRUE));
+    fprintf(out, "    From: %s\n", znfp1(&pkt->from));
+    fprintf(out, "    To  : %s\n", znfp2(&pkt->to));
     fprintf(out, "    Date: %s\n", date(NULL, &pkt->time));
     fprintf(out, "    Baud: %d\n", pkt->baud);
     fprintf(out, "    Prod: %02x %02x\n", pkt->product_h, pkt->product_l);
