@@ -2,12 +2,12 @@
 /*****************************************************************************
  * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
  *
- * $Id: ftn2rfc.c,v 4.28 1997/10/26 10:42:32 mj Exp $
+ * $Id: ftn2rfc.c,v 4.53 1999/10/04 08:18:23 mj Exp $
  *
  * Convert FTN mail packets to RFC mail and news batches
  *
  *****************************************************************************
- * Copyright (C) 1990-1997
+ * Copyright (C) 1990-1999
  *  _____ _____
  * |     |___  |   Martin Junius             FIDO:      2:2452/110
  * | | | |   | |   Radiumstr. 18             Internet:  mj@fido.de
@@ -40,15 +40,12 @@
 
 
 #define PROGRAM 	"ftn2rfc"
-#define VERSION 	"$Revision: 4.28 $"
-#define CONFIG		CONFIG_GATE
+#define VERSION 	"$Revision: 4.53 $"
+#define CONFIG		DEFAULT_CONFIG_GATE
 
 
 
-/*
- * Prototypes
- */
-void	check_chrs		(char *);
+/* Prototypes */
 char   *get_from		(Textlist *, Textlist *);
 char   *get_reply_to		(Textlist *);
 char   *get_to			(Textlist *);
@@ -64,16 +61,14 @@ void	usage			(void);
 
 
 
-/*
- * Command line options
- */
+/* Command line options */
+int n_flag = FALSE;
 int t_flag = FALSE;
 
 char in_dir[MAXPATH];
 
 
-/*
- * X-FTN flags
+/* X-FTN flags
  *    f    X-FTN-From
  *    t    X-FTN-To
  *    T    X-FTN-Tearline
@@ -81,8 +76,7 @@ char in_dir[MAXPATH];
  *    V    X-FTN-Via
  *    D    X-FTN-Domain
  *    S    X-FTN-Seen-By
- *    P    X-FTN-Path
- */
+ *    P    X-FTN-Path      */
 int x_ftn_f = FALSE;
 int x_ftn_t = FALSE;
 int x_ftn_T = FALSE;
@@ -93,12 +87,9 @@ int x_ftn_S = FALSE;
 int x_ftn_P = FALSE;
 
 
-/*
- * TrackerMail
- */
+/* TrackerMail */
 static char *tracker_mail_to = NULL;
 
-
 /* MSGID handling */
 static int no_unknown_msgid_zones    = FALSE;
 static int no_messages_without_msgid = FALSE;
@@ -131,21 +122,15 @@ static int kill_split = FALSE;
 /* Write single news article files, not one big news batch */
 static int single_articles = FALSE;
 
+/* Charset stuff */
+static char *default_charset_def = NULL;
+static char *default_charset_out = NULL;
+static char *netmail_charset_def = NULL;
+static char *netmail_charset_out = NULL;
+static int netmail_charset_use_1st = FALSE;
 
-
-/*
- * check_chrs() --- Check for ^ACHARSET / ^ACHRS
- */
-void check_chrs(char *buf)
-{
-    /*
-     * Look for `^ACHARSET ...' or `^ACHRS ...'
-     */
-    if(!strncmp(buf+1, "CHARSET: ", 9))
-	charset_set(buf + 9);
-    else if(!strncmp(buf+1, "CHRS: ", 6))
-	charset_set(buf + 6);
-}
+/* String to add to news Path header */
+static char *news_path_tail = "not-for-mail";
 
 
 
@@ -258,6 +243,24 @@ int check_8bit(Textlist *tl)
     return FALSE;
 }
 
+#ifdef AI_5
+/*
+ * Check for 8bit characters in any string
+ */
+int check_8bit_s(char *s, int len)
+{
+    char *p;
+
+    if(!s)
+	return FALSE;
+
+    for(p=s; (int)(p-s)<len && *p!='\0'; p++)
+	if(*p & 0x80)
+	    return TRUE;
+
+    return FALSE;
+}
+#endif
 
 
 /*
@@ -313,7 +316,9 @@ int unpack(FILE *pkt_file, Packet *pkt)
     int rfc_lvl, rfc_lines;
     char *split_line;
     int cvt8 = 0;			/* AREA_8BIT | AREA_QP */
-    
+    char *cs_def, *cs_in, *cs_out;	/* Charset def, in(=FTN), out(=RFC) */
+    char *cs_save;
+
 
     /*
      * Initialize
@@ -345,7 +350,6 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	x_orig_to[0] = 0;
 	tl_clear(&theader);
 	tl_clear(&tbody);
-	charset_reset();			/* Reset ^ACHRS char set */
 
 	/*
 	 * Read message header
@@ -394,6 +398,9 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	/* Retrieve address information from kludges for NetMail */
 	if(body.area == NULL)
 	{
+	    /* Don't use point address from packet for Netmail */
+	    msg.node_from.point = 0;
+	    msg.node_to  .point = 0;
 	    /* Retrieve complete address from kludges */
 	    kludge_pt_intl(&body, &msg, FALSE);
 	    msg.node_orig = msg.node_from;
@@ -405,8 +412,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		/* No * Origin line address, use header */
 		node_invalid(&msg.node_orig);
 	}
-	debug(7, "FIDO sender (from/origin): %s",
-	      node_to_asc(&msg.node_orig, TRUE));
+	debug(7, "FIDO sender (from/origin): %s", znfp1(&msg.node_orig));
 
 	/*
 	 * strip_crlf() all kludge and RFC lines
@@ -483,7 +489,14 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	 * Check for 8bit characters in message body. If none are
 	 * found, don't use the quoted-printable encoding or 8bit.
 	 */
+#ifndef AI_5
 	if(!check_8bit(&body.body))
+#else
+	if( !(check_8bit(&body.body)                         ||
+	      check_8bit_s(body.origin, sizeof(body.origin)) ||
+	      check_8bit_s(body.tear, sizeof(body.tear))     ||
+	      check_8bit_s(msg.subject, sizeof(msg.subject))) )
+#endif
 	    cvt8 &= ~(AREA_QP | AREA_8BIT);
 	
 	/*
@@ -501,17 +514,55 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    if(strieq(p, "quoted-printable")) 
 		cvt8 = AREA_QP;
 	}
+	debug(5, "cvt8:%s%s",
+	      (cvt8 & AREA_8BIT ? " 8bit" : ""),
+	      (cvt8 & AREA_QP   ? " quoted-printable" : ""));
 	
 	/*
 	 * Convert message body
 	 */
-	lines = 0;
 
+	/* charset input (=FTN message) and output (=RFC message) */
+	cs_save = NULL;
+	cs_def  = NULL;
+	cs_in   = NULL;
+	cs_out  = NULL;
+	
+	if(area)				/* EchoMail -> News */
+	{
+	    if(area->charset)
+	    {
+		cs_save = strsave(area->charset);
+		cs_def  = strtok(cs_save, ":");
+		s       = strtok(NULL, ":");
+		cs_out  = strtok(NULL, ":");
+	    }
+	}
+	else					/* NetMail -> Mail */
+	{
+	    cs_def  = netmail_charset_def;
+	    cs_out  = netmail_charset_out;
+	}
+	/* defaults */
+	if(!cs_def)
+	    cs_def = default_charset_def;
+	if(!cs_def)
+	    cs_def = CHARSET_STDFTN;
+	if(!cs_out)
+	    cs_out = default_charset_out;
+	if(cvt8==0 || !cs_out)
+	    cs_out = CHARSET_STD7BIT;
+	
 	if( (p = kludge_get(&body.kludge, "CHRS", NULL)) )
-	    charset_set(p);
+	    cs_in = charset_chrs_name(p);
 	else if( (p = kludge_get(&body.kludge, "CHARSET", NULL)) )
-	    charset_set(p);
+	    cs_in = charset_chrs_name(p);
+	if(!cs_in)
+	    cs_in = cs_def;
+	charset_set_in_out(cs_in, cs_out);
+	/**FIXME: if ERROR is returned, use first matching alias for cs_in**/
 
+	/* ^ARFC level and line break flag */
 	rfc_lvl   = 0;
 	rfc_lines = FALSE;
 	if( (p = kludge_get(&body.kludge, "RFC", NULL)) )
@@ -526,14 +577,21 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		rfc_lines = TRUE;
 	}
 	    
+	lines = 0;
 	for(pl=body.body.first; pl; pl=pl->next)
 	{
 	    p = pl->line;
 	    if(*p == '\001')			/* Kludge in message body */
-		check_chrs(p);			/* ^ACHRS / ^ACHARSET */
+	    {
+		if(strnieq(p + 1, "CHRS: ", 6))
+		    cs_in = charset_chrs_name(p + 6);
+		if(strnieq(p + 1, "CHARSET: ", 9))
+		    cs_in = charset_chrs_name(p + 9);
+		/**FIXME: change in/out charset if needed**/
+	    }
 	    else				/* Normal line */
 	    {
-		msg_xlate_line(buffer, sizeof(buffer), p, cvt8);
+		msg_xlate_line(buffer, sizeof(buffer), p, cvt8 & AREA_QP);
 		if(rfc_lines)
 		{
 		    tl_append(&tbody, buffer);
@@ -548,6 +606,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	/*
 	 * Convert FTN from/to addresses to RFCAddr struct
 	 */
+	charset_set_in_out(cs_in, CHARSET_STD7BIT);
 	addr_from = rfcaddr_from_ftn(msg.name_from, &msg.node_orig);
 	addr_to   = rfcaddr_from_ftn(msg.name_to,   &msg.node_to  );
 
@@ -576,14 +635,13 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    /* ^A SPLIT */
 	    if( (p = kludge_get(&body.kludge, "SPLIT", NULL)) )
 	    {
-		log("skipping split message, origin=%s",
-		    node_to_asc(&msg.node_orig, TRUE));
+		log("skipping split message, origin=%s", znfp1(&msg.node_orig));
 		continue;
 	    }
 	}
 	
 	/*
-	 * If -g flag is set for area and message seems to comme from
+	 * If -g flag is set for area and message seems to come from
 	 * another gateway, skip it.
 	 */
 	if(area && (area->flags & AREA_NOGATE))
@@ -591,7 +649,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    if(msgbody_rfc_from)
 	    {
 		log("skipping message from gateway, area %s, origin=%s",
-		    area->area, node_to_asc(&msg.node_orig, TRUE));
+		    area->area, znfp1(&msg.node_orig));
 		continue;
 	    }
 	
@@ -600,7 +658,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	       !strnicmp(p, "GIGO", 4)                         )
 	    {
 		log("skipping message from gateway (GIGO), area %s, origin=%s",
-		    area->area, node_to_asc(&msg.node_orig, TRUE));
+		    area->area, znfp1(&msg.node_orig));
 		continue;
 	    }
 
@@ -608,10 +666,9 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    if( (p = kludge_get(&body.kludge, "X-FZ-SPLIT", NULL)) )
 	    {
 		log("skipping message from gateway (X-FZ-SPLIT), area %s, origin=%s",
-		    area->area, node_to_asc(&msg.node_orig, TRUE));
+		    area->area, znfp1(&msg.node_orig));
 		continue;
 	    }
-	    
 	}
 
 	/*
@@ -622,12 +679,20 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    Alias *a;
 
 	    debug(7, "Checking for alias: %s",
-		  rfcaddr_to_asc(&addr_from, TRUE));
-	    a = alias_lookup(&msg.node_orig, NULL, addr_from.real);
+		  s_rfcaddr_to_asc(&addr_from, TRUE));
+	    /**FIXME: why _strict()?**/
+	    a = alias_lookup_strict(&msg.node_orig, NULL, addr_from.real);
 	    if(a)
 	    {
-		debug(7, "Alias found: %s %s %s", a->username,
-		      node_to_asc(&a->node, FALSE), a->fullname);
+		if(a->userdom)
+		{
+		    debug(7, "Alias found: %s@%s %s %s", a->username, a->userdom,
+			  znfp1(&a->node), a->fullname);
+		    BUF_COPY(addr_from.addr, a->userdom);
+		}
+		else
+		    debug(7, "Alias found: %s %s %s", a->username,
+		          znfp1(&a->node), a->fullname);
 		BUF_COPY(addr_from.user, a->username);
 #ifdef ALIASES_ARE_LOCAL
 		BUF_COPY(addr_from.addr, cf_fqdn());
@@ -639,36 +704,47 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    Alias *a;
 	    
 	    debug(7, "Checking for alias: %s",
-		  rfcaddr_to_asc(&addr_to, TRUE));
-	    a = alias_lookup(cf_addr(), NULL, addr_to.real);
+		  s_rfcaddr_to_asc(&addr_to, TRUE));
+	    /**FIXME: why _strict()?**/
+	    a = alias_lookup_strict(&msg.node_to, NULL, addr_to.real);
 	    if(a)
 	    {
-		debug(7, "Alias found: %s %s %s", a->username,
-		      node_to_asc(&a->node, FALSE), a->fullname);
-		BUF_COPY(mail_to, a->username);
+		if(a->userdom)
+		{
+		    debug(7, "Alias (AI2) found: %s@%s %s \"%s\"",
+			  a->username, a->userdom,
+			  znfp1(&a->node), a->fullname);
+		    /*BUF_COPY(addr_to.addr, a->userdom);*/
+		    BUF_COPY  (mail_to, a->username);
+		    BUF_APPEND(mail_to, "@");
+		    BUF_APPEND(mail_to, a->userdom);
+		}
+		else
+		{
+		    debug(7, "Alias (old) found: %s %s \"%s\"",
+			  a->username,
+		          znfp1(&a->node), a->fullname);
+		    BUF_COPY(mail_to, a->username);
+		}
 	    }
 	    else
 		BUF_COPY(mail_to, addr_to.user);
 	}
 
-	/*
-	 * Special handling for -t flag (insecure):
-	 *   Messages with To line will be bounced
-	 */
+	/* Special handling for -t flag (insecure): Messages with To
+	 * line will be bounced */
 	if(area==NULL && t_flag && msgbody_rfc_to)
 	{
 	    debug(1, "Insecure message with To line");
 	    log("BOUNCE: insecure mail from %s",
-		rfcaddr_to_asc(&addr_from, TRUE));
+		s_rfcaddr_to_asc(&addr_from, TRUE));
 	    bounce_mail("insecure", &addr_from, &msg, msgbody_rfc_to, &tbody);
 	    continue;
 	}
 	    
-	/*
-	 * There are message trackers out there in FIDONET. Obviously
-	 * they can't handling addressing the gateway so we send mail
-	 * from "MsgTrack..." etc. to TrackerMail.
-	 */
+	/* There are message trackers out there in FIDONET. Most of
+	 * they can't handle addressing the gateway so we send mail
+	 * from "MsgTrack..." etc. to TrackerMail.  */
 	if(tracker_mail_to)
 	    if(   !strnicmp(addr_from.user, "MsgTrack", 8)
 	       || !strnicmp(addr_from.user, "Reflex_Netmail_Policeman", 24)
@@ -685,15 +761,13 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    }
 
 
-	thisdomain   = ftn_to_inet(cf_addr(),      TRUE);
-	uplinkdomain = ftn_to_inet(&msg.node_from, TRUE);
+	thisdomain   = s_ftn_to_inet(cf_addr(),      TRUE);
+	uplinkdomain = s_ftn_to_inet(&msg.node_from, TRUE);
 	origindomain = msg.node_orig.zone != -1
-	    ? ftn_to_inet(&msg.node_orig, TRUE) : FTN_INVALID_DOMAIN;
+	    ? s_ftn_to_inet(&msg.node_orig, TRUE) : FTN_INVALID_DOMAIN;
 
-	/*
-	 * Bounce mail from nodes not registered in HOSTS,
-	 * but allow mail to local users.
-	 */
+	/* Bounce mail from nodes not registered in HOSTS, but allow
+	 * mail to local users.  */
 	if(addr_is_restricted() && !ignore_hosts &&
 	   area==NULL && msgbody_rfc_to && !addr_is_domain(msgbody_rfc_to))
 	{
@@ -704,9 +778,9 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    {
 		/* Not registered in HOSTS */
 		debug(1, "Not a registered node: %s",
-		      node_to_asc(&msg.node_orig, FALSE));
+		      znfp1(&msg.node_orig));
 		log("BOUNCE: mail from unregistered %s",
-		    rfcaddr_to_asc(&addr_from, TRUE));
+		    s_rfcaddr_to_asc(&addr_from, TRUE));
 		bounce_mail("restricted", &addr_from, &msg,
 			    msgbody_rfc_to, &tbody);
 		continue;
@@ -716,9 +790,9 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    if(h->flags & HOST_DOWN)
 	    {
 		debug(1, "Registered node is down: %s",
-		      node_to_asc(&msg.node_orig, FALSE));
+		      znfp1(&msg.node_orig));
 		log("BOUNCE: mail from down %s",
-		    rfcaddr_to_asc(&addr_from, TRUE));
+		    s_rfcaddr_to_asc(&addr_from, TRUE));
 		bounce_mail("down", &addr_from, &msg,
 			    msgbody_rfc_to, &tbody);
 		continue;
@@ -737,7 +811,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		{
 		    debug(1, "Message with address in mail_to: %s", mail_to);
 		    log("BOUNCE: mail from %s with address in to field: %s",
-			rfcaddr_to_asc(&addr_from, TRUE), mail_to           );
+			s_rfcaddr_to_asc(&addr_from, TRUE), mail_to           );
 		    bounce_mail("addrinto",
 				&addr_from, &msg, msgbody_rfc_to, &tbody);
 		    continue;
@@ -763,7 +837,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		/* Addressed to `UUCP' or `GATEWAY', but no To: line */
 		debug(1, "Message to `UUCP' or `GATEWAY' without To line");
 		log("BOUNCE: mail from %s without To line",
-		    rfcaddr_to_asc(&addr_from, TRUE));
+		    s_rfcaddr_to_asc(&addr_from, TRUE));
 		bounce_mail("noto", &addr_from, &msg, msgbody_rfc_to, &tbody);
 		continue;
 	    }
@@ -777,9 +851,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		BUF_APPEND(mail_to, cf_fqdn());
 	}
 
-	/*
-	 * Construct string for From: header line
-	 */
+	/* Construct string for From: header line */
 	if(msgbody_rfc_from) {
 	    RFCAddr rfc;
 	    
@@ -789,22 +861,18 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    
 	    addr_from = rfc;
 	}
-	from_line = rfcaddr_to_asc(&addr_from, TRUE);
+	from_line = s_rfcaddr_to_asc(&addr_from, TRUE);
 
-	/*
-	 * Construct Reply-To line
-	 */
+	/* Construct Reply-To line */
 	reply_to_line = msgbody_rfc_reply_to;
 	
-	/*
-	 * Construct string for To:/X-Comment-To: header line
-	 */
+	/* Construct string for To:/X-Comment-To: header line */
 	if(msgbody_rfc_to) {
 	    if(strchr(msgbody_rfc_to, '(') || strchr(msgbody_rfc_to, '<') ||
 	       !*addr_to.real || uucp_flag                                   )
-		to_line = buf_sprintf("%s", msgbody_rfc_to);
+		to_line = s_printf("%s", msgbody_rfc_to);
 	    else
-		to_line = buf_sprintf("%s (%s)", msgbody_rfc_to, addr_to.real);
+		to_line = s_printf("%s (%s)", msgbody_rfc_to, addr_to.real);
 	}
 	else {
 	    if(area)
@@ -815,30 +883,26 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		   !strcmp(addr_to.user, "*")      )
 		    to_line = NULL;
 		else
-		    to_line = buf_sprintf("(%s)", addr_to.real);
+		    to_line = s_printf("(%s)", addr_to.real);
 	    }
 	    else
-		to_line = buf_sprintf("%s", mail_to);
+		to_line = s_printf("%s", mail_to);
 	}
 
-	/*
-	 * Construct Cc/Bcc header lines
-	 */
+	/* Construct Cc/Bcc header lines */
 	cc_line  = msgbody_rfc_to ? msgbody_rfc_cc  : NULL;
 	bcc_line = msgbody_rfc_to ? msgbody_rfc_bcc : NULL;
 	
-	/*
-	 * Construct Message-ID and References header lines
-	 */
+	/* Construct Message-ID and References header lines */
 	id_line  = NULL;
 	ref_line = NULL;
 	
 	if( (p = kludge_get(&body.kludge, "ORIGID", NULL)) )
-	    id_line = msgid_convert_origid(p, FALSE);
+	    id_line = s_msgid_convert_origid(p, FALSE);
 	else if( (p = kludge_get(&body.kludge, "Message-Id", NULL)) )
-	    id_line = msgid_convert_origid(p, FALSE);
+	    id_line = s_msgid_convert_origid(p, FALSE);
 	else if( (p = kludge_get(&body.kludge, "RFC-Message-ID", NULL)) )
-	    id_line = msgid_convert_origid(p, FALSE);
+	    id_line = s_msgid_convert_origid(p, FALSE);
 	if(!id_line)
 	{
 	    int id_zone;
@@ -850,11 +914,11 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		    log("MSGID: %s, not gated", p);
 		    continue;
 		}
-		id_line = msgid_fido_to_rfc(p, &id_zone);
+		id_line = s_msgid_fido_to_rfc(p, &id_zone);
 		if(no_unknown_msgid_zones)
 		    if(id_zone>=-1 && !cf_zones_check(id_zone))
 		    {
-			log("MSGID %s: unknown zone, not gated", p);
+			log("MSGID %s: malformed or unknown zone, not gated", p);
 			continue;
 		    }
 	    }
@@ -865,7 +929,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		    log("MSGID: none, not gated");
 		    continue;
 		}
-		id_line = msgid_default(&msg);
+		id_line = s_msgid_default(&msg);
 	    }
 	}
 	/* Can't happen, but who knows ... ;-) */
@@ -876,10 +940,10 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	}
 	
 	if( (p = kludge_get(&body.kludge, "ORIGREF", NULL)) )
-	    ref_line = msgid_convert_origid(p, FALSE);
+	    ref_line = s_msgid_convert_origid(p, FALSE);
 	if(!ref_line)
 	    if( (p = kludge_get(&body.kludge, "REPLY", NULL)) )
-		ref_line = msgid_fido_to_rfc(p, NULL);
+		ref_line = s_msgid_fido_to_rfc(p, NULL);
 
 	/* ^AGATEWAY */
 	gateway = kludge_get(&body.kludge, "GATEWAY", NULL);
@@ -894,8 +958,8 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    log("MAIL: %s -> %s", from_line, to_line);
 	    
 	    tl_appendf(&theader,
-			     "From %s %s\n", rfcaddr_to_asc(&addr_from, FALSE),
-			     date("%a %b %d %H:%M:%S %Y", NULL) );
+		       "From %s %s\n", s_rfcaddr_to_asc(&addr_from, FALSE),
+		       date(DATE_FROM, NULL) );
 	    tl_appendf(&theader,
 		"Received: by %s (FIDOGATE %s)\n",
 		thisdomain, version_global()                            );
@@ -907,12 +971,13 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	{
 	    if(!strcmp(thisdomain, uplinkdomain))	/* this == uplink */
 		tl_appendf(&theader,
-			   "Path: %s!%s!not-for-mail\n",
-			   thisdomain, origindomain );
+			   "Path: %s!%s!%s\n",
+			   thisdomain, origindomain, news_path_tail);
 	    else
 		tl_appendf(&theader,
-			   "Path: %s!%s!%s!not-for-mail\n",
-			   thisdomain, uplinkdomain, origindomain );
+			   "Path: %s!%s!%s!%s\n",
+			   thisdomain, uplinkdomain, origindomain,
+			   news_path_tail                         );
 	}
 
 	/* Common header */
@@ -920,7 +985,11 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	tl_appendf(&theader, "From: %s\n", from_line);
 	if(reply_to_line)
 	    tl_appendf(&theader, "Reply-To: %s\n", reply_to_line);
+#ifndef AI_5
 	msg_xlate_line(buffer, sizeof(buffer), msg.subject, 0);
+#else
+	msg_xlate_line(buffer, sizeof(buffer), msg.subject, cvt8);
+#endif
 	tl_appendf(&theader, "Subject: %s\n", buffer);
 	tl_appendf(&theader, "Message-ID: %s\n", id_line);
 	
@@ -940,7 +1009,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    /* FTN ReturnReceiptRequest -> Return-Receipt-To */
 	    if(msg.attr & MSG_RRREQ)
 		tl_appendf(&theader, "Return-Receipt-To: %s\n",
-				 rfcaddr_to_asc(&addr_from, FALSE)   );
+				 s_rfcaddr_to_asc(&addr_from, FALSE)   );
 	}
 	else 					/* News */
 	{
@@ -961,7 +1030,11 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	if(use_origin_for_organization && body.origin)
 	{
 	    strip_crlf(body.origin);
+#ifndef AI_5
 	    msg_xlate_line(buffer, sizeof(buffer), body.origin, 0);
+#else
+	    msg_xlate_line(buffer, sizeof(buffer), body.origin, cvt8);
+#endif
 	    if((p = strrchr(buffer, '(')))
 		*p = 0;
 	    strip_space(buffer);
@@ -972,7 +1045,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	}
 	else
 	{
-	    tl_appendf(&theader, "Organization: %s\n", cf_organization() );
+	    tl_appendf(&theader, "Organization: %s\n", cf_p_organization() );
 	}
 	tl_appendf(&theader, "Lines: %d\n", lines);
 	if(gateway)
@@ -986,22 +1059,30 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	{
 	    if(x_ftn_f)
 		tl_appendf(&theader, "X-FTN-From: %s @ %s\n",
-			   addr_from.real, node_to_asc(&msg.node_orig,TRUE));
+			   addr_from.real, znfp1(&msg.node_orig));
 	    if(x_ftn_t)
 		tl_appendf(&theader, "X-FTN-To: %s @ %s\n",
-			   addr_to.real, node_to_asc(&msg.node_to,TRUE));
+			   addr_to.real, znfp1(&msg.node_to));
 	}
 
 	if(x_ftn_T  &&  body.tear && !strncmp(body.tear, "--- ", 4))
 	{
 	    strip_crlf(body.tear);
+#ifndef AI_5
 	    msg_xlate_line(buffer, sizeof(buffer), body.tear, 0);
+#else
+	    msg_xlate_line(buffer, sizeof(buffer), body.tear, cvt8);
+#endif
 	    tl_appendf(&theader, "X-FTN-Tearline: %s\n", buffer+4);
 	}
 	if(!use_origin_for_organization  &&  x_ftn_O  &&  body.origin)
 	{
 	    strip_crlf(body.origin);
+#ifndef AI_5
 	    msg_xlate_line(buffer, sizeof(buffer), body.origin, 0);
+#else
+	    msg_xlate_line(buffer, sizeof(buffer), body.origin, cvt8);
+#endif
 	    p = buffer + strlen(" * Origin: ");
 	    while(is_blank(*p))
 		p++;
@@ -1036,28 +1117,25 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	if(split_line)
 	    tl_appendf(&theader, "X-SPLIT: %s\n", split_line);
 
-	/*
-	 * MIME header
-	 */
+	/* MIME header */
 	tl_appendf(&theader, "MIME-Version: 1.0\n");
 	tl_appendf(&theader, "Content-Type: text/plain; charset=%s\n",
-		   cvt8 ? "iso-8859-1" : "us-ascii");
+		   cvt8 ? cs_out : CHARSET_STD7BIT);
 	tl_appendf(&theader, "Content-Transfer-Encoding: %s\n",
 		   cvt8 ? ((cvt8 & AREA_QP) ? "quoted-printable" : "8bit")
 		        : "7bit");
 
-	/*
-	 * Add extra headers
-	 */
+	if(cs_save)
+	    xfree(cs_save);
+	
+	/* Add extra headers */
 	if(area)
 	    for(pl=area->x_hdr.first; pl; pl=pl->next)
 		tl_appendf(&theader, "%s\n", pl->line);
 	    
 	tl_appendf(&theader, "\n");
 
-	/*
-	 * Write header and message body to output file
-	 */
+	/* Write header and message body to output file */
 	if(area) {
 	    if(!mail_file('n'))
 		if(mail_open('n') == ERROR)
@@ -1091,12 +1169,13 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    mail_close('m');
 	}
 
+	tmps_freeall();
     } /**while(type == MSG_TYPE)**/
 
     if(mail_file('n')) 
 	mail_close('n');
-    
-    return ret;
+
+    TMPS_RETURN(ret);
 }
 
 
@@ -1135,38 +1214,49 @@ int unpack_file(char *pkt_name)
     Packet pkt;
     FILE *pkt_file;
 
-    /*
-     * Open packet and read header
-     */
+    /* Open packet and read header */
     pkt_file = fopen(pkt_name, R_MODE);
     if(!pkt_file) {
 	log("$ERROR: can't open packet %s", pkt_name);
-	rename_bad(pkt_name);
-	return OK;
+	if(n_flag)
+	    return ERROR;
+	else
+	{
+	    rename_bad(pkt_name);
+	    return OK;
+	}
     }
     if(pkt_get_hdr(pkt_file, &pkt) == ERROR)
     {
 	log("ERROR: reading header from %s", pkt_name);
-	rename_bad(pkt_name);
-	return OK;
+	if(n_flag)
+	    return ERROR;
+	else
+	{
+	    rename_bad(pkt_name);
+	    return OK;
+	}
     }
     
-    /*
-     * Unpack it
-     */
+    /* * Unpack it */
     log("packet %s (%ldb) from %s to %s", pkt_name, check_size(pkt_name),
-	node_to_asc(&pkt.from, TRUE), node_to_asc(&pkt.to, TRUE) );
+	znfp1(&pkt.from), znfp2(&pkt.to) );
     
-    if(unpack(pkt_file, &pkt) == -1) 
+    if(unpack(pkt_file, &pkt) == ERROR) 
     {
 	log("ERROR: processing %s", pkt_name);
-	rename_bad(pkt_name);
-	return OK;
+	if(n_flag)
+	    return ERROR;
+	else
+	{
+	    rename_bad(pkt_name);
+	    return OK;
+	}
     }
     
     fclose(pkt_file);
     
-    if (unlink(pkt_name)) {
+    if(!n_flag && unlink(pkt_name)==ERROR) {
 	log("$ERROR: can't unlink packet %s", pkt_name);
 	rename_bad(pkt_name);
 	return OK;
@@ -1195,17 +1285,16 @@ void usage(void)
     fprintf(stderr, "usage:   %s [-options] [packet ...]\n\n", PROGRAM);
     fprintf(stderr, "\
 options: -1 --single-articles         write single news articles, not batch\n\
-         -i --ignore-hosts            do not bounce unknown host\n\
-	 -t --insecure                process insecure packets\n\
          -I --in-dir name             set input packet directory\n\
+         -i --ignore-hosts            do not bounce unknown host\n\
          -l --lock-file               create lock file while processing\n\
+         -n --no-remove               don't remove/rename input packet file\n\
+	 -t --insecure                process insecure packets\n\
          -x --exec-program name       exec program after processing\n\
 \n\
 	 -v --verbose                 more verbose\n\
 	 -h --help                    this help\n\
          -c --config name             read config file (\"\" = none)\n\
-	 -L --lib-dir name            set lib directory\n\
-	 -S --spool-dir name          set spool directory\n\
 	 -a --addr Z:N/F.P            set FTN address\n\
 	 -u --uplink-addr Z:N/F.P     set FTN uplink address\n");
     
@@ -1218,12 +1307,11 @@ options: -1 --single-articles         write single news articles, not batch\n\
 
 int main(int argc, char **argv)
 {
-    int c;
+    int c, ret;
     char *execprog = NULL;
     int l_flag=FALSE;
     char *I_flag=NULL;
     char *c_flag=NULL;
-    char *S_flag=NULL, *L_flag=NULL;
     char *a_flag=NULL, *u_flag=NULL;
     char *pkt_name;
     char *p;
@@ -1232,17 +1320,16 @@ int main(int argc, char **argv)
     static struct option long_options[] =
     {
 	{ "single-articles", 0, 0, '1'},/* Write single article files */
-	{ "ignore-hosts", 0, 0, 'i'},	/* Do not bounce unknown hosts */
-	{ "insecure",     0, 0, 't'},	/* Toss insecure packets */
 	{ "in-dir",       1, 0, 'I'},	/* Set inbound packets directory */
+	{ "ignore-hosts", 0, 0, 'i'},	/* Do not bounce unknown hosts */
 	{ "lock-file",    0, 0, 'l'},	/* Create lock file while processing */
+	{ "no-remove",    0, 0, 'n'},	/* Don't remove/rename packet file */
+	{ "insecure",     0, 0, 't'},	/* Toss insecure packets */
 	{ "exec-program", 1, 0, 'x'},	/* Exec program after processing */
 
 	{ "verbose",      0, 0, 'v'},	/* More verbose */
 	{ "help",         0, 0, 'h'},	/* Help */
 	{ "config",       1, 0, 'c'},	/* Config file */
-	{ "spool-dir",    1, 0, 'S'},	/* Set FIDOGATE spool directory */
-	{ "lib-dir",      1, 0, 'L'},	/* Set FIDOGATE lib directory */
 	{ "addr",         1, 0, 'a'},	/* Set FIDO address */
 	{ "uplink-addr",  1, 0, 'u'},	/* Set FIDO uplink address */
 	{ 0,              0, 0, 0  }
@@ -1254,7 +1341,7 @@ int main(int argc, char **argv)
     cf_initialize();
 
 
-    while ((c = getopt_long(argc, argv, "1itI:lx:vhc:S:L:a:u:",
+    while ((c = getopt_long(argc, argv, "1itI:lnx:vhc:a:u:",
 			    long_options, &option_index     )) != EOF)
 	switch (c) {
 	/***** ftn2rfc options *****/
@@ -1262,22 +1349,26 @@ int main(int argc, char **argv)
 	    /* Write single article files */
 	    single_articles = TRUE;
 	    break;
-	case 'i':
-	    /* Don't bounce unknown hosts */
-	    ignore_hosts = TRUE;
-	    break;
-	case 't':
-	    /* Insecure */
-	    t_flag = TRUE;
-	    break;
 	case 'I':
 	    /* Inbound packets directory */
 	    I_flag = optarg;
 	    break;
+	case 'i':
+	    /* Don't bounce unknown hosts */
+	    ignore_hosts = TRUE;
+	    break;
         case 'l':
             /* Lock file */
 	    l_flag = TRUE;
             break;
+	case 'n':
+	    /* Don't remove/rename input packet file */
+	    n_flag = TRUE;
+	    break;
+	case 't':
+	    /* Insecure */
+	    t_flag = TRUE;
+	    break;
         case 'x':
             /* Exec program after unpack */
             execprog = optarg;
@@ -1294,12 +1385,6 @@ int main(int argc, char **argv)
 	case 'c':
 	    c_flag = optarg;
 	    break;
-	case 'S':
-	    S_flag = optarg;
-	    break;
-	case 'L':
-	    L_flag = optarg;
-	    break;
 	case 'a':
 	    a_flag = optarg;
 	    break;
@@ -1312,46 +1397,26 @@ int main(int argc, char **argv)
 	    break;
 	}
 
-    /*
-     * Read config file
-     */
-    if(L_flag)				/* Must set libdir beforehand */
-	cf_set_libdir(L_flag);
+    /* Read config file */
     cf_read_config_file(c_flag ? c_flag : CONFIG);
 
-    /*
-     * Process config options
-     */
-    if(L_flag)
-	cf_set_libdir(L_flag);
-    if(S_flag)
-	cf_set_spooldir(S_flag);
+    /* Process config options */
     if(a_flag)
 	cf_set_addr(a_flag);
     if(u_flag)
 	cf_set_uplink(u_flag);
 
+    cf_i_am_a_gateway_prog();
     cf_debug();
     
-    /*
-     * Process local options
-     */
-    if(I_flag)
-    {
-	str_expand_name(in_dir, sizeof(in_dir), I_flag);
-	BUF_COPY3(mail_dir, in_dir, "/", INDIR_MAIL);
-	BUF_COPY3(news_dir, in_dir, "/", INDIR_NEWS);
-    }
-    else 
-    {
-	BUF_COPY3(in_dir  , cf_spooldir(), "/", t_flag ? INSECUREDIR : INDIR);
-	BUF_COPY3(mail_dir, in_dir, "/", INDIR_MAIL);
-	BUF_COPY3(news_dir, in_dir, "/", INDIR_NEWS);
-    }
-    
-    /*
-     * Process optional config statements
-     */
+    /* Process local options */
+    BUF_EXPAND(in_dir, I_flag ? I_flag : cf_p_pinbound());
+
+    /* Initialize mail_dir[], news_dir[] output directories */
+    BUF_EXPAND(mail_dir, DEFAULT_OUTRFC_MAIL);
+    BUF_EXPAND(news_dir, DEFAULT_OUTRFC_NEWS);
+
+    /* Process optional config statements */
     if(cf_get_string("DotNames", TRUE))
     {
 	debug(8, "config: DotNames");
@@ -1463,21 +1528,54 @@ int main(int argc, char **argv)
 	rfcaddr_mode(m);
 	debug(8, "config: RFCAddrMode %d", m);
     }
+    if( (p = cf_get_string("DefaultCharset", TRUE)) )
+    {
+	debug(8, "config: DefaultCharset %s", p);
+	default_charset_def = strtok(p, ":");
+	strtok(NULL, ":");
+	default_charset_out = strtok(NULL, ":");
+    }
+    if( (p = cf_get_string("NetMailCharset", TRUE)) )
+    {
+	debug(8, "config: NetMailCharset %s", p);
+	netmail_charset_def = strtok(p, ":");
+	strtok(NULL, ":");
+	netmail_charset_out = strtok(NULL, ":");
+    }
+    if( (p = cf_get_string("NewsPathTail", TRUE)) )
+    {
+	/* <FIDOGATE_CONFIG>
+	 * <CMD>     NewsPathTail
+	 * <PARA>    STRING
+	 * <DEFAULT> "not-for-mail"
+	 * <DESC>    The STRING which FIDOGATE's ftn2rfc adds to the
+	 *           Path header, normally "not-for-mail". If gated
+	 *           messages are not generally exported to the Usenet,
+	 *           setting it to "fidogate!not-for-mail" makes
+	 *           the INN newsfeeds entry easier and less error-prone.
+	 */
+	debug(8, "config: NewsPathTail %s", p);
+	news_path_tail = p;
+    }
+    if(cf_get_string("NetMailCharsetUse1st", TRUE))
+    {
+	debug(8, "config: NetMailCharsetUse1st");
+	netmail_charset_use_1st = TRUE;
+    }
     
-    /*
-     * Init various modules
-     */
+    /* Init various modules */
     areas_init();
     hosts_init();
     alias_init();
+    charset_init();
 
-    /*
-     * If called with -l lock option, try to create lock FILE
-     */
+    /* If called with -l lock option, try to create lock FILE */
     if(l_flag)
 	if(lock_program(PROGRAM, NOWAIT) == ERROR)
 	    exit(EXIT_BUSY);
 
+    ret = EXIT_OK;
+
     if(optind >= argc)
     {
 	/* process packet files in input directory */
@@ -1491,37 +1589,43 @@ int main(int argc, char **argv)
 	}
 
 	for(pkt_name=dir_get(TRUE); pkt_name; pkt_name=dir_get(FALSE))
-	    unpack_file(pkt_name);
+	{
+	    if(unpack_file(pkt_name) != OK)
+		ret = EXIT_ERROR;
+	    tmps_freeall();
+	}
 
 	dir_close();
     }
     else
     {
-	/*
-	 * Process packet files on command line
-	 */
+	/* Process packet files on command line */
 	for(; optind<argc; optind++)
-	    unpack_file(argv[optind]);
+	{
+	    if(unpack_file(argv[optind]) != OK)
+		ret = EXIT_ERROR;
+	    tmps_freeall();
+	}
     }
     
 
-    /*
-     * Execute given command, if option -x set.
-     */
+    /* Execute given command, if option -x set.  */
     if(execprog)
     {
-	int ret;
+	int retx;
 
-	str_expand_name(buffer, sizeof(buffer), execprog);
+	BUF_EXPAND(buffer, execprog);
 	debug(4, "Command: %s", buffer);
-	ret = run_system(buffer);
-	debug(4, "Exit code=%d", ret);
+	retx = run_system(buffer);
+	debug(4, "Exit code=%d", retx);
+	if(retx != EXIT_OK)
+	    ret = EXIT_ERROR;
     }
     
     if(l_flag)
 	unlock_program(PROGRAM);
     
-    exit(EX_OK);
+    exit(ret);
 
     /**NOT REACHED**/
     return 1;
