@@ -2,12 +2,12 @@
 /*****************************************************************************
  * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
  *
- * $Id: ftnpack.c,v 4.15 1997/08/13 19:11:37 mj Exp $
+ * $Id: ftnpack.c,v 4.25 1999/07/23 21:30:13 mj Exp $
  *
  * Pack output packets of ftnroute for Binkley outbound (ArcMail)
  *
  *****************************************************************************
- * Copyright (C) 1990-1997
+ * Copyright (C) 1990-1999
  *  _____ _____
  * |     |___  |   Martin Junius             FIDO:      2:2452/110
  * | | | |   | |   Radiumstr. 18             Internet:  mj@fido.de
@@ -40,8 +40,8 @@
 
 
 #define PROGRAM 	"ftnpack"
-#define VERSION 	"$Revision: 4.15 $"
-#define CONFIG		CONFIG_MAIN
+#define VERSION 	"$Revision: 4.25 $"
+#define CONFIG		DEFAULT_CONFIG_MAIN
 
 
 
@@ -84,13 +84,13 @@ static Packing *packing_last  = NULL;
  * Prototypes
  */
 int	parse_pack		(char *);
-ArcProg*parse_arc		(char *);
-void	new_arc			(int, char *);
-Packing*packing_parse_line	(char *);
+ArcProg *parse_arc		(char *);
+void	new_arc			(int);
+Packing *packing_parse_line	(char *);
 int	packing_do_file		(char *);
 void	packing_init		(char *);
 char   *arcmail_name		(Node *, char *);
-char   *pkttime_name		(char *);
+char   *packing_pkt_name		(char *, char *);
 int	arcmail_search		(char *);
 int	do_arcmail		(char *, Node *, Node *, PktDesc *,
 				 FILE *, char *, char *);
@@ -121,10 +121,14 @@ int parse_pack(char *s)
 	return PACK_FLO;
     if(!stricmp(s, "dirpack"))
 	return PACK_DIR;
+    if(!stricmp(s, "dirmove"))
+	return PACK_MOVE;
     if(!stricmp(s, "arc"))
 	return PACK_ARC;
     if(!stricmp(s, "prog"))
 	return PACK_PROG;
+    if(!stricmp(s, "progn"))
+	return PACK_PROGN;
     
     return ERROR;
 }
@@ -149,25 +153,24 @@ ArcProg *parse_arc(char *s)
 /*
  * Define new archiver/program
  */
-void new_arc(int cmd, char *name)
+void new_arc(int cmd)
 {
-    char *p;
+    char *name, *prog;
     ArcProg *a;
     
-    p = xstrtok(NULL, " \t");
-    if(!p)
+    name = xstrtok(NULL, " \t");
+    prog = xstrtok(NULL, " \t");
+    if(!name || !prog)
     {
-	log("packing: missing command for arc/prog %s", name);
+	log("packing: missing argument for arc/prog definition", name);
 	return;
     }
     
-    /*
-     * Create new entry and put into list
-     */
+    /* Create new entry and put into list */
     a = (ArcProg *)xmalloc(sizeof(ArcProg));
     a->pack = cmd;
     a->name = strsave(name);
-    a->prog = strsave(p);
+    a->prog = strsave(prog);
     a->next = NULL;
 
     if(arcprog_first)
@@ -176,7 +179,7 @@ void new_arc(int cmd, char *name)
 	arcprog_first      = a;
     arcprog_last = a;
 	
-    debug(15, "routing: pack=%c name=%s prog=%s",
+    debug(15, "packing: pack=%c name=%s prog=%s",
 	  a->pack, a->name, a->prog             );
 }
 
@@ -195,62 +198,62 @@ Packing *packing_parse_line(char *buf)
     LON lon;
     int cmd;
 
-    /*
-     * Command
-     */
+    /* Command */
     p = xstrtok(buf, " \t");
     if(!p)
-    {
 	return NULL;
-    }
     if(strieq(p, "include"))
     {
 	p = xstrtok(NULL, " \t");
 	packing_do_file(p);
 	return NULL;
     }
-    
     if((cmd = parse_pack(p)) == ERROR)
     {
 	log("packing: unknown command %s", p);
 	return NULL;
     }
 
-    /*
-     * Packer name
-     */
-    p = xstrtok(NULL, " \t");
-    if(!p)
+    /* Definition of new archiver/program */
+    if(cmd==PACK_ARC || cmd==PACK_PROG || cmd==PACK_PROGN)
     {
-	log("packing: packer name/directory argument missing");
+	new_arc(cmd);
 	return NULL;
     }
 
-    if(cmd==PACK_ARC || cmd==PACK_PROG)
+    /* Directory argument */
+    if(cmd==PACK_DIR || cmd==PACK_MOVE)
     {
-	/* Definition of new archiver/program */
-	new_arc(cmd, p);
-	return NULL;
-    }
-    if(cmd == PACK_DIR)
-    {
-	/* Directory name for "dirpack" command */
-	dir = strsave(p);
 	p = xstrtok(NULL, " \t");
+	if(!p)
+	{
+	    log("packing: directory argument missing");
+	    return NULL;
+	}
+	dir = strsave(p);
     }
     else
 	dir = NULL;
 
-    /* Archiver/program name */
-    if((a = parse_arc(p)) == NULL)
+    /* Archiver name argument */
+    if(cmd!=PACK_MOVE)
     {
-	log("packing: unknown archiver/program %s", p);
-	return NULL;
+	p = xstrtok(NULL, " \t");
+	if(!p)
+	{
+	    log("packing: archiver name argument missing");
+	    return NULL;
+	}
+	if((a = parse_arc(p)) == NULL)
+	{
+	    log("packing: unknown archiver/program %s", p);
+	    return NULL;
+	}
     }
-	
-    /*
-     * List of nodes follows, using "*" or "all" wildcard pattern
-     */
+    else
+	a = NULL;
+    
+    /* List of nodes follows */
     node_invalid(&old);
     old.zone = cf_zone();
     lon_init(&lon);
@@ -276,9 +279,7 @@ Packing *packing_parse_line(char *buf)
 	p = xstrtok(NULL, " \t");
     }
 
-    /*
-     * Create new entry and put into list
-     */
+    /* Create new entry and put into list */
     r = (Packing *)xmalloc(sizeof(Packing));
     r->pack  = cmd;
     r->dir   = dir;
@@ -287,7 +288,7 @@ Packing *packing_parse_line(char *buf)
     r->next  = NULL;
 	
     debug(15, "packing: pack=%c dir=%s arc=%s",
-	  r->pack, r->dir ? r->dir : "", r->arc->name);
+	  r->pack, r->dir ? r->dir : "", (r->arc ? r->arc->name : NULL));
     lon_debug(15, "packing: nodes=", &r->nodes, TRUE);
 
     return r;
@@ -343,6 +344,7 @@ char *arcmail_name(Node *node, char *dir)
 #ifndef AMIGADOS_4D_OUTBOUND
     int d1, d2;
 #endif
+    size_t rest;
 
     cf_set_zone(node->zone);
 
@@ -356,13 +358,14 @@ char *arcmail_name(Node *node, char *dir)
     else
     {
 	/* Outbound dir + zone dir */
-	BUF_COPY2(buf, cf_outbound(), "/");
+	BUF_COPY2(buf, cf_p_btbasedir(), "/");
 	if((base = cf_zones_out(node->zone)) == NULL)
 	    return NULL;
 	BUF_APPEND(buf, base);
 	BUF_APPEND(buf, "/");
     }
     base = buf + strlen(buf);
+    rest = sizeof(buf) - strlen(buf);
 
     /*
      * Get weekday archive extension
@@ -375,8 +378,8 @@ char *arcmail_name(Node *node, char *dir)
      * Create name of archive file
      */
 #ifdef AMIGADOS_4D_OUTBOUND
-    sprintf(base, "%d.%d.%d.%d.%s", node->zone, node->net,
-	    node->node, node->point, wk);
+    str_printf(base, rest, "%d.%d.%d.%d.%s", node->zone, node->net,
+	       node->node, node->point, wk);
 #else    
     if(node->point)
     {
@@ -384,17 +387,17 @@ char *arcmail_name(Node *node, char *dir)
 	d2 = (cf_main_addr()->point - node->point) & 0xffff;
 
 	if(dir)
-	    sprintf(base, "%04x%04x.%s", d1, d2, wk );
+	    str_printf(base, rest, "%04x%04x.%s", d1, d2, wk );
 	else
-	    sprintf(base, "%04x%04x.pnt/%04x%04x.%s",
-		    node->net, node->node, d1, d2, wk );
+	    str_printf(base, rest, "%04x%04x.pnt/%04x%04x.%s",
+		       node->net, node->node, d1, d2, wk );
     }
     else
     {
 	d1 = (cf_main_addr()->net  - node->net ) & 0xffff;
 	d2 = (cf_main_addr()->node - node->node) & 0xffff;
 	
-	sprintf(base, "%04x%04x.%s", d1, d2, wk);
+	str_printf(base, rest, "%04x%04x.%s", d1, d2, wk);
     }
 #endif /**AMIGADOS_4D_OUTBOUND**/
     
@@ -404,25 +407,26 @@ char *arcmail_name(Node *node, char *dir)
 
 
 /*
- * Return time coded packet name for adding to archive
+ * Return packet name for adding to archive
  */
-char *pkttime_name(char *name)
+char *packing_pkt_name(char *dir, char *name)
 {
     static char buf[MAXPATH];
     char *p;
     
 #if 0
-    /* Return nnnnnnnn.pkt in out_dir[] */
-    sprintf(buf, "%s/%08ld.pkt", out_dir, sequencer(SEQ_PACK));
+    /* Return nnnnnnnn.pkt in dir */
+    str_printf(buf, sizeof(buf), "%s/%08ld.pkt",
+	       (dir ? dir : out_dir), sequencer(DEFAULT_SEQ_PACK));
 #endif
 
-    /* Same base name in out_dir[] */
+    /* Same base name in dir */
     p = strrchr(name, '/');
     if(!p)				/* Just to be sure */
 	p = name;
     else
 	p++;
-    BUF_COPY3(buf, out_dir, "/", p);
+    BUF_COPY3(buf, (dir ? dir : out_dir), "/", p);
     
     return buf;
 }
@@ -496,7 +500,7 @@ int do_arcmail(char *name, Node *arcnode, Node *flonode,
     int ret, newfile;
     
     arcn = arcmail_name(arcnode, dir);
-    pktn = ffx_flag ? name : pkttime_name(name);
+    pktn = ffx_flag ? name : packing_pkt_name(NULL, name);
     if(!arcn)
 	return ERROR;
     if(arcmail_search(arcn) == ERROR)
@@ -508,7 +512,7 @@ int do_arcmail(char *name, Node *arcnode, Node *flonode,
 	return ERROR;
     }
     
-    debug(4, "Archiving %s for %s arc", name, node_to_asc(arcnode, TRUE));
+    debug(4, "Archiving %s for %s arc", name, znfp1(arcnode));
     debug(4, "    Packet  name: %s", pktn);
     debug(4, "    Archive name: %s", arcn);
 
@@ -538,7 +542,7 @@ int do_arcmail(char *name, Node *arcnode, Node *flonode,
 	}
     }
     
-    sprintf(buffer, prog, arcn, pktn);
+    str_printf(buffer, sizeof(buffer), prog, arcn, pktn);
     debug(4, "Command: %s", buffer);
     ret = run_system(buffer);
     debug(4, "Exit code=%d", ret);
@@ -551,7 +555,7 @@ int do_arcmail(char *name, Node *arcnode, Node *flonode,
     chmod(arcn, PACKET_MODE);
     if(unlink(pktn) == -1)
 	log("$ERROR: can't remove %s", pktn);
-    if(!dir && newfile)
+    if(!dir)
 	return bink_attach(flonode, '#', arcn,
 			   flav_to_asc(desc->flav), FALSE );
 
@@ -583,9 +587,9 @@ int do_noarc(char *name, Node *flonode,
     if(fp == NULL)
     {
 	log("ERROR: can't open outbound packet for %s",
-	    node_to_asc(&desc->to, TRUE)      );
+	    znfp1(&desc->to)      );
 	fclose(pkt_file);
-	return ERROR;
+	TMPS_RETURN(ERROR);
     }
     
     tl_init(&tl);
@@ -604,7 +608,7 @@ int do_noarc(char *name, Node *flonode,
 	    log("$ERROR reading input packet %s", name);
 	    pkt_close();
 	    fclose(pkt_file);
-	    return ERROR;
+	    TMPS_RETURN(ERROR);
 	}
 	/* Read message body */
 	type = pkt_get_body(pkt_file, &tl);
@@ -613,7 +617,7 @@ int do_noarc(char *name, Node *flonode,
 	    log("$ERROR: reading input packet %s", name);
 	    pkt_close();
 	    fclose(pkt_file);
-	    return ERROR;
+	    TMPS_RETURN(ERROR);
 	}
 
 	/* File attaches */
@@ -632,10 +636,10 @@ int do_noarc(char *name, Node *flonode,
 				      flav_to_asc(desc->flav), FALSE );
 		    if(ret == ERROR)
 			log("ERROR: file attach %s for %s failed",
-			    fa_name, node_to_asc(&desc->to, TRUE));
+			    fa_name, znfp1(&desc->to));
 		    else
 			log("file attach %s (%ldb) for %s",
-			    fa_name, sz, node_to_asc(&desc->to, TRUE));
+			    fa_name, sz, znfp1(&desc->to));
 	    }
 	    /* File attachments from inbound directory */
 	    else if(file_attach_dir[0])
@@ -648,10 +652,10 @@ int do_noarc(char *name, Node *flonode,
 				      flav_to_asc(desc->flav), FALSE );
 		    if(ret == ERROR)
 			log("ERROR: file attach %s for %s failed",
-			    msg.subject, node_to_asc(&desc->to, TRUE));
+			    msg.subject, znfp1(&desc->to));
 		    else
 			log("file attach %s (%ldb) for %s",
-			    msg.subject, sz, node_to_asc(&desc->to, TRUE));
+			    msg.subject, sz, znfp1(&desc->to));
 		}
 		else
 		    log("file attach %s: no such file", msg.subject);
@@ -666,7 +670,7 @@ int do_noarc(char *name, Node *flonode,
 	    log("$ERROR: writing packet %s", pkt_name());
 	    pkt_close();
 	    fclose(pkt_file);
-	    return ERROR;
+	    TMPS_RETURN(ERROR);
 	}
 	/* Write message body */
 	tl_print(&tl, fp);
@@ -677,23 +681,25 @@ int do_noarc(char *name, Node *flonode,
 	    log("$ERROR: writing packet %s", pkt_name());
 	    pkt_close();
 	    fclose(pkt_file);
-	    return ERROR;
+	    TMPS_RETURN(ERROR);
 	}
+	
+	tmps_freeall();
     } /**while**/
 
     fclose(pkt_file);
     if(pkt_close() != OK)
     {
 	log("$ERROR: can't close outbound packet");
-	return ERROR;
+	TMPS_RETURN(ERROR);
     }
     if(unlink(name) != OK)
     {
 	log("$ERROR: can't remove packet %s", name);
-	return ERROR;
+	TMPS_RETURN(ERROR);
     }
     
-    return OK;
+    TMPS_RETURN(OK);
 }
 
 
@@ -701,13 +707,13 @@ int do_noarc(char *name, Node *flonode,
 /*
  * Call program for packet file
  */
-int do_prog(char *name, PktDesc *desc, char *prog)
+int do_prog(char *name, PktDesc *desc, Packing *pack)
 {
     int ret;
     
     debug(4, "Processing %s", name);
 
-    sprintf(buffer, prog, name);
+    str_printf(buffer, sizeof(buffer), pack->arc->prog, name);
     debug(4, "Command: %s", buffer);
     ret = run_system(buffer);
     debug(4, "Exit code=%d", ret);
@@ -716,8 +722,9 @@ int do_prog(char *name, PktDesc *desc, char *prog)
         log("ERROR: %s failed, exit code=%d", buffer, ret);
         return ERROR;
     }
-    if(unlink(name) == -1)
-        log("$ERROR: can't remove %s", name);
+    if(pack->arc->pack != PACK_PROGN)
+	if(unlink(name)==ERROR)
+	    log("$ERROR: can't remove %s", name);
 
     return OK;
 }
@@ -774,7 +781,7 @@ int do_pack(PktDesc *desc, char *name, FILE *file, Packing *pack)
     if((!ffx_flag || !node_eq(&arcnode, &ffx_node)) &&
        bink_bsy_create(&arcnode, NOWAIT) == ERROR)
     {
-	debug(1, "%s busy, skipping", node_to_asc(&arcnode, TRUE));
+	debug(1, "%s busy, skipping", znfp1(&arcnode));
 	if(file)
 	    fclose(file);
 	return OK;			/* This is o.k. */
@@ -782,7 +789,7 @@ int do_pack(PktDesc *desc, char *name, FILE *file, Packing *pack)
     if(!node_eq(&arcnode, &flonode) &&
        bink_bsy_create(&flonode, NOWAIT) == ERROR)
     {
-	debug(1, "%s busy, skipping", node_to_asc(&flonode, TRUE));
+	debug(1, "%s busy, skipping", znfp1(&flonode));
 	if(file)
 	    fclose(file);
 	bink_bsy_delete(&arcnode);
@@ -790,7 +797,7 @@ int do_pack(PktDesc *desc, char *name, FILE *file, Packing *pack)
     }
        
     /* Do the various pack functions */
-    if(pack->arc->pack == PACK_ARC)
+    if(pack->arc && pack->arc->pack==PACK_ARC)
     {
 	if( ffx_flag ||
 	    (desc->flav!=FLAV_CRASH && pack->arc->prog) )
@@ -798,17 +805,17 @@ int do_pack(PktDesc *desc, char *name, FILE *file, Packing *pack)
 	    if(pack->pack == PACK_ROUTE)
 		log("archiving packet (%ldb) for %s via %s arc (%s)",
 		    check_size(name),
-		    node_to_asc(&desc->to,TRUE), node_to_asc(&arcnode,TRUE),
+		    znfp1(&desc->to), znfp2(&arcnode),
 		    pack->arc->name );
 	    else if(pack->pack == PACK_FLO)
 		log("archiving packet (%ldb) for %s via %s flo (%s)",
 		    check_size(name),
-		    node_to_asc(&desc->to,TRUE), node_to_asc(&flonode,TRUE),
+		    znfp1(&desc->to), znfp2(&flonode),
 		    pack->arc->name );
 	    else
 		log("archiving packet (%ldb) for %s (%s)",
 		    check_size(name),
-		    node_to_asc(&desc->to,TRUE), pack->arc->name );
+		    znfp1(&desc->to), pack->arc->name );
 
 	    ret = do_arcmail(name, &arcnode, &flonode, desc,
 			     file, pack->arc->prog, NULL);
@@ -816,7 +823,7 @@ int do_pack(PktDesc *desc, char *name, FILE *file, Packing *pack)
 	else
 	{
 	    log("packet (%ldb) for %s (noarc)",
-		check_size(name), node_to_asc(&desc->to,TRUE));
+		check_size(name), znfp1(&desc->to));
 	    ret = do_noarc(name, &desc->to, desc, file, NULL);
 	}
     }
@@ -824,7 +831,9 @@ int do_pack(PktDesc *desc, char *name, FILE *file, Packing *pack)
     {
 	if(file)
 	    fclose(file);
-	ret = do_prog(name, desc, pack->arc->prog);
+	log("packet (%ldb) for %s (%s)",
+	    check_size(name), znfp1(&desc->to), pack->arc->name);
+	ret = do_prog(name, desc, pack);
     }
     
     /* Delete BSY file */
@@ -845,6 +854,7 @@ int do_dirpack(PktDesc *desc, char *name, FILE *file, Packing *pack)
 {
     int ret = OK;
     Node arcnode, flonode;
+    char *pktn;
     
     arcnode = desc->to;
     flonode = desc->to;
@@ -857,25 +867,34 @@ int do_dirpack(PktDesc *desc, char *name, FILE *file, Packing *pack)
     if(!ffx_flag)
 	if(bink_bsy_create(&arcnode, NOWAIT) == ERROR)
 	{
-	    debug(1, "%s busy, skipping", node_to_asc(&arcnode, TRUE));
+	    debug(1, "%s busy, skipping", znfp1(&arcnode));
 	    if(file)
 		fclose(file);
 	    return OK;			/* This is o.k. */
 	}
 
-    /* Do the various pack functions */
-    if(pack->arc->pack == PACK_ARC)
+    /* dirpack */
+    if(pack->pack==PACK_DIR && pack->arc && pack->arc->pack==PACK_ARC)
     {
 	if(pack->arc->prog)
 	{
 	    log("archiving packet (%ldb) for %s (%s) in %s",
-		check_size(name),
-		node_to_asc(&desc->to,TRUE), pack->arc->name, pack->dir);
+		check_size(name), znfp1(&desc->to), pack->arc->name, pack->dir);
 
 	    ret = do_arcmail(name, &arcnode, &flonode, desc,
 			     file, pack->arc->prog, pack->dir);
 	}
     }
+
+    /* dirmove */
+    if(pack->pack==PACK_MOVE)
+    {
+	log("moving packet (%ldb) for %s to %s",
+	    check_size(name), znfp1(&desc->to), pack->dir);
+
+	pktn = packing_pkt_name(pack->dir, name);
+	ret = do_noarc(name, &flonode, desc, file, pktn);
+    }
     
     /* Delete BSY file */
     if(!ffx_flag)
@@ -895,6 +914,7 @@ int do_packing(char *name, FILE *fp, Packet *pkt)
     PktDesc pktdesc;
     Packing *r;
     LNode *p;
+    int ret;
 
     if(pkt_flag || ffx_flag)
     {
@@ -912,11 +932,11 @@ int do_packing(char *name, FILE *fp, Packet *pkt)
 	/* Parse description from filename */
 	desc = parse_pkt_name(name, &pkt->from, &pkt->to);
 	if(desc == NULL)
-	    return ERROR;
+	    TMPS_RETURN(ERROR);
     }
     
     debug(2, "Packet: from=%s to=%s grade=%c type=%c flav=%c",
-	  node_to_asc(&desc->from, TRUE), node_to_asc(&desc->to, TRUE),
+	  znfp1(&desc->from), znfp2(&desc->to),
 	  desc->grade, desc->type, desc->flav);
 
     /*
@@ -927,12 +947,14 @@ int do_packing(char *name, FILE *fp, Packet *pkt)
 	    if(node_match(&desc->to, &p->node))
 	    {
 		debug(3, "packing: pack=%c dir=%s arc=%s",
-		      r->pack, r->dir ? r->dir : "", r->arc->name);
-		return r->dir ? do_dirpack(desc, name, fp, r)
-		              : do_pack   (desc, name, fp, r);
+		      r->pack, r->dir ? r->dir : "",
+		      (r->arc ? r->arc->name : NULL)      );
+		ret = r->dir ? do_dirpack(desc, name, fp, r)
+		             : do_pack   (desc, name, fp, r);
+		TMPS_RETURN(ret);
 	    }
 
-    return OK;
+    TMPS_RETURN(OK);
 }
 
 
@@ -959,12 +981,12 @@ int do_file(char *pkt_name)
 	pkt_file = fopen(pkt_name, R_MODE);
 	if(!pkt_file) {
 	    log("$ERROR: can't open packet %s", pkt_name);
-	    return severe_error;
+	    TMPS_RETURN(severe_error);
 	}
 	if(pkt_get_hdr(pkt_file, &pkt) == ERROR)
 	{
 	    log("ERROR: reading header from %s", pkt_name);
-	    return severe_error;
+	    TMPS_RETURN(severe_error);
 	}
     }
     
@@ -972,10 +994,10 @@ int do_file(char *pkt_name)
     if(do_packing(pkt_name, pkt_file, &pkt) == ERROR) 
     {
 	log("ERROR: processing %s", pkt_name);
-	return severe_error;
+	TMPS_RETURN(severe_error);
     }
 
-    return OK;
+    TMPS_RETURN(OK);
 }
 
 
@@ -1037,8 +1059,6 @@ options: -B --binkley NAME            set Binkley outbound directory\n\
 	 -v --verbose                 more verbose\n\
 	 -h --help                    this help\n\
          -c --config NAME             read config file (\"\" = none)\n\
-	 -L --lib-dir NAME            set lib directory\n\
-	 -S --spool-dir NAME          set spool directory\n\
 	 -a --addr Z:N/F.P            set FTN address\n\
 	 -u --uplink-addr Z:N/F.P     set FTN uplink address\n");
     
@@ -1056,7 +1076,6 @@ int main(int argc, char **argv)
     int l_flag = FALSE;
     char *I_flag=NULL, *O_flag=NULL, *B_flag=NULL, *p_flag=NULL;
     char *c_flag=NULL;
-    char *S_flag=NULL, *L_flag=NULL;
     char *a_flag=NULL, *u_flag=NULL;
     char *pkt_name;
     char pattern[16];
@@ -1078,8 +1097,6 @@ int main(int argc, char **argv)
 	{ "verbose",      0, 0, 'v'},	/* More verbose */
 	{ "help",         0, 0, 'h'},	/* Help */
 	{ "config",       1, 0, 'c'},	/* Config file */
-	{ "spool-dir",    1, 0, 'S'},	/* Set FIDOGATE spool directory */
-	{ "lib-dir",      1, 0, 'L'},	/* Set FIDOGATE lib directory */
 	{ "addr",         1, 0, 'a'},	/* Set FIDO address */
 	{ "uplink-addr",  1, 0, 'u'},	/* Set FIDO uplink address */
 	{ 0,              0, 0, 0  }
@@ -1091,7 +1108,7 @@ int main(int argc, char **argv)
     cf_initialize();
 
 
-    while ((c = getopt_long(argc, argv, "B:f:F:g:I:O:lp:m:Pvhc:S:L:a:u:",
+    while ((c = getopt_long(argc, argv, "B:f:F:g:I:O:lp:m:Pvhc:a:u:",
 			    long_options, &option_index     )) != EOF)
 	switch (c) {
 	/***** ftnpack options *****/
@@ -1142,12 +1159,6 @@ int main(int argc, char **argv)
 	case 'c':
 	    c_flag = optarg;
 	    break;
-	case 'S':
-	    S_flag = optarg;
-	    break;
-	case 'L':
-	    L_flag = optarg;
-	    break;
 	case 'a':
 	    a_flag = optarg;
 	    break;
@@ -1163,19 +1174,13 @@ int main(int argc, char **argv)
     /*
      * Read config file
      */
-    if(L_flag)				/* Must set libdir beforehand */
-	cf_set_libdir(L_flag);
     cf_read_config_file(c_flag ? c_flag : CONFIG);
 
     /*
      * Process config options
      */
     if(B_flag)
-	cf_set_outbound(B_flag);
-    if(L_flag)
-	cf_set_libdir(L_flag);
-    if(S_flag)
-	cf_set_spooldir(S_flag);
+	cf_s_btbasedir(B_flag);
     if(a_flag)
 	cf_set_addr(a_flag);
     if(u_flag)
@@ -1195,16 +1200,10 @@ int main(int argc, char **argv)
     /*
      * Process local options
      */
-    if(I_flag)
-	BUF_EXPAND(in_dir, I_flag);
-    else 
-	BUF_COPY3(in_dir, cf_spooldir(), "/", TOSS_OUT);
-    if(O_flag)
-	BUF_EXPAND(out_dir, O_flag);
-    else
-	BUF_COPY3(out_dir, cf_spooldir(), "/", TOSS_PACK);
+    BUF_EXPAND(in_dir , I_flag ? I_flag : DEFAULT_TOSS_ROUTE);
+    BUF_EXPAND(out_dir, O_flag ? O_flag : DEFAULT_TOSS_PACK);
 
-    packing_init(p_flag ? p_flag : PACKING);
+    packing_init(p_flag ? p_flag : cf_p_packing() );
     passwd_init();
 
     /* Install signal/exit handlers */
@@ -1257,6 +1256,7 @@ int main(int argc, char **argv)
 		    ret = EXIT_ERROR;
 		    break;
 		}
+		tmps_freeall();
 	    }
 	    dir_close();
 	}
@@ -1281,11 +1281,14 @@ int main(int argc, char **argv)
 	
 	/* Process packet files on command line */
 	for(; optind<argc; optind++)
+	{
 	    if(do_file(argv[optind]) == ERROR)
 	    {
 		ret = EXIT_ERROR;
 		break;
 	    }
+	    tmps_freeall();
+	}
 	
 	/* Lock file */
 	if(l_flag)
