@@ -2,12 +2,12 @@
 /*****************************************************************************
  * FIDOGATE --- Gateway UNIX Mail/News <-> FIDO NetMail/EchoMail
  *
- * $Id: ftntoss.c,v 4.20 1997/08/17 13:13:22 mj Exp $
+ * $Id: ftntoss.c,v 4.36 1999/04/03 13:29:04 mj Exp $
  *
  * Toss FTN NetMail/EchoMail
  *
  *****************************************************************************
- * Copyright (C) 1990-1997
+ * Copyright (C) 1990-1999
  *  _____ _____
  * |     |___  |   Martin Junius             FIDO:      2:2452/110
  * | | | |   | |   Radiumstr. 18             Internet:  mj@fido.de
@@ -39,8 +39,8 @@
 
 
 #define PROGRAM 	"ftntoss"
-#define VERSION 	"$Revision: 4.20 $"
-#define CONFIG		CONFIG_MAIN
+#define VERSION 	"$Revision: 4.36 $"
+#define CONFIG		DEFAULT_CONFIG_MAIN
 
 
 
@@ -49,6 +49,10 @@
  */
 void	addtoseenby_init	(void);
 void	zonegate_init		(void);
+#ifdef AI_3
+void	deleteseenby_init	(void);
+void	deletepath_init		(void);
+#endif
 int	lon_to_kludge		(Textlist *, char *, LON *);
 int	lon_to_kludge_sorted	(Textlist *, char *, LON *);
 int	toss_echomail		(Message *, MsgBody *, LON *, LON *, LON *);
@@ -118,7 +122,8 @@ int kill_dupe     = FALSE;		/* config: KillDupe */
 int kill_nomsgid  = FALSE;		/* config: KillNoMSGID */
 int kill_old      = FALSE;		/* config: KillOld */
 int echomail4d    = FALSE;		/* config: EchoMail4D */ 
-int no_empty_path = FALSE;		/* config: NoEmptyPath */ 
+int no_empty_path = FALSE;		/* config: NoEmptyPath */
+int add_other_aka = FALSE;		/* config: AddOtherAKAs */ 
 
 /* Values checking for old messages */
 static double max_history = 14;		/* Max. number of days entry stays
@@ -197,7 +202,97 @@ void addtoseenby_init(void)
     }
 }
 
+
+
+#ifdef AI_3
+/*
+ * DeleteSeenBy list
+ */
+typedef struct st_deleteseenby
+{
+    LON deleteseenby;				/* Nodes to delete from SEEN-BY */
+    struct st_deleteseenby *next;
+}
+DeleteSeenBy;
+
+static DeleteSeenBy *deleteseenby_first = NULL;
+static DeleteSeenBy *deleteseenby_last  = NULL;
+
+
+
+/*
+ * Init DeleteSeenBy list from config file
+ */
+void deleteseenby_init(void)
+{
+    char *s;
+    DeleteSeenBy *p;
     
+    for(s = cf_get_string("DeleteSeenBy",TRUE);
+	s && *s;
+	s = cf_get_string("DeleteSeenBy",FALSE) )
+    {
+	debug(8, "config: deleteseenby %s", s);
+	
+	p = (DeleteSeenBy *)xmalloc(sizeof(DeleteSeenBy));
+	p->next = NULL;
+	lon_init(&p->deleteseenby);
+	lon_add_string(&p->deleteseenby, s);
+
+	if(deleteseenby_first)
+	    deleteseenby_last->next = p;
+	else
+	    deleteseenby_first = p;
+	deleteseenby_last = p;
+    }
+}
+    
+
+
+/*
+ * DeletePath list
+ */
+typedef struct st_deletepath
+{
+    LON deletepath;				/* Nodes to delete from PATH */
+    struct st_deletepath *next;
+}
+DeletePath;
+
+static DeletePath *deletepath_first = NULL;
+static DeletePath *deletepath_last  = NULL;
+
+
+
+/*
+ * Init DeletePath list from config file
+ */
+void deletepath_init(void)
+{
+    char *s;
+    DeletePath *p;
+    
+    for(s = cf_get_string("DeletePath",TRUE);
+	s && *s;
+	s = cf_get_string("DeletePath",FALSE) )
+    {
+	debug(8, "config: deletepath %s", s);
+	
+	p = (DeletePath *)xmalloc(sizeof(DeletePath));
+	p->next = NULL;
+	lon_init(&p->deletepath);
+	lon_add_string(&p->deletepath, s);
+
+	if(deletepath_first)
+	    deletepath_last->next = p;
+	else
+	    deletepath_first = p;
+	deletepath_last = p;
+    }
+}
+#endif
+
+
 
 /*
  * Zonegate list
@@ -325,7 +420,7 @@ int lon_to_kludge_sorted(Textlist *tl, char *text, LON *lon)
 	    
 	    if(strlen(buffer)+strlen(s)+1 > MAX_LENGTH)
 	    {
-		strncat0(buffer, "\r\n", sizeof(buffer));
+		BUF_APPEND(buffer, "\r\n");
 		tl_append(tl, buffer);
 		
 		node_invalid(&old);
@@ -370,7 +465,7 @@ int toss_echomail(Message *msg, MsgBody *body,
 	is_saved = FALSE;
 
 	debug(7, "toss_echomail(): message for %s",
-	      node_to_asc(&p->node, TRUE));
+	      znfp1(&p->node));
 	
 	/* Check for msg addressed to zonegate */
 	if(zonegate_first)
@@ -438,7 +533,7 @@ int kludge_to_lon(Textlist *tl, LON *lon)
 	    p++;
 	while(*p && is_space(*p))
 	    p++;
-	strncpy0(buffer, p, sizeof(buffer));
+	BUF_COPY(buffer, p);
 	strip_crlf(buffer);
 	/* Tokenize and convert to node addresses */
 	node_invalid(&old);
@@ -642,6 +737,59 @@ int do_4dpoint(LON *seenby, LON *path, Node *from, Node *addr)
 
 
 
+#ifdef AI_3
+/*
+ * SEEN-BY remove processing
+ */
+void do_deleteseenby(LON *seenby)
+{
+    LNode *p;
+    LON *ln;
+
+    ln = &deleteseenby_first->deleteseenby;
+
+    lon_debug(9, "do_deleteseenby(): List of SEEN-BYs for removing: ",
+	      ln, FALSE);
+
+    for(p=ln->first; p; p=p->next)
+    {
+        if( lon_search(seenby, &p->node) )
+	{
+	    debug(7, "do_deleteseenby(): Found SEEN-BY for removing: %s", 
+		  znf1(&p->node));
+	    lon_remove(seenby, &p->node);
+	}
+    }
+}
+
+
+
+/*
+ * PATH remove processing
+ */
+void do_deletepath(LON *path)
+{
+    LNode *p;
+    LON *ln;
+
+    ln = &deleteseenby_first->deleteseenby;
+    
+    lon_debug(9, "do_deletepath(): List of PATH for removing: ", ln, FALSE);
+
+    for(p=ln->first; p; p=p->next)
+    {
+        if( lon_search(path, &p->node) )
+	{
+	    debug(7, "do_deletepath(): Found PATH for removing: %s",
+		  znf1(&p->node));
+	    lon_remove(path, &p->node);
+	}
+    }
+}
+#endif
+
+
+
 /*
  * Process EchoMail message
  */
@@ -650,20 +798,20 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
     AreasBBS *area;
     AddToSeenBy *addto;
     LON seenby, path, new;
-    int ret, n;
+    int ret, n, z;
+    Node *node;
 
     /*
      * Lookup area and set zone
      */
-    strncpy0(buffer, body->area + strlen("AREA:"), sizeof(buffer));
+    BUF_COPY(buffer, body->area + strlen("AREA:"));
     strip_crlf(buffer);
     debug(5, "EchoMail AREA: %s", buffer);
     
     if( (area = areasbbs_lookup(buffer)) == NULL )
     {
 	/* Unknown area */
-	log("unknown area %s from %s", buffer,
-	    node_to_asc(&msg->node_from, TRUE) );
+	log("unknown area %s from %s", buffer, znfp1(&msg->node_from) );
 	msgs_unknown++;
 	if(!kill_unknown)
 	    return do_bad_msg(msg, body);
@@ -705,7 +853,7 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	    if(kill_nomsgid)
 	    {
 		log("no ^AMSGID treated as dupe from %s: %s",
-		    node_to_asc(&msg->node_from, TRUE), area->area);
+		    znfp1(&msg->node_from), area->area);
 		msgs_dupe++;
 		if(!kill_dupe)
 		    return do_bad_msg(msg, body);
@@ -716,7 +864,7 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	    if(msg_parse_origin(body->origin, &msg->node_orig) == ERROR)
 	    {
 		log("invalid * Origin treated as dupe from %s: %s",
-		    node_to_asc(&msg->node_from, TRUE), area->area);
+		    znfp1(&msg->node_from), area->area);
 		msgs_dupe++;
 		if(!kill_dupe)
 		    return do_bad_msg(msg, body);
@@ -729,9 +877,9 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	    crc32_compute(msg->name_to  , strlen(msg->name_to  ));
 	    crc32_compute(msg->subject  , strlen(msg->subject  ));
 
-	    sprintf(buffer, "%s NOMSGID: %s %s %08lx",
-		    area->area, node_to_asc(&msg->node_orig, TRUE),
-		    date("%y%m%d %H%M%S", &msg->date), crc32_value() );
+	    str_printf(buffer, sizeof(buffer), "%s NOMSGID: %s %s %08lx",
+		       area->area, znfp1(&msg->node_orig),
+		       date("%y%m%d %H%M%S", &msg->date), crc32_value() );
 	    msgid = buffer;
 	}
 
@@ -741,7 +889,7 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	    if(msg->date < exp_sec)
 	    {
 		log("message too old, treated as dupe: %s origin %s date %s",
-		    area->area, node_to_asc(&msg->node_orig, TRUE),
+		    area->area, znfp1(&msg->node_orig),
 		    date(DATE_FTS_0001, &msg->date)                          );
 		msgs_dupe++;
 		if(!kill_dupe)
@@ -754,7 +902,7 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	if(hi_test(msgid))
 	{
 	    /* Dupe! */
-	    log("dupe from %s: %s", node_to_asc(&msg->node_from, TRUE), msgid);
+	    log("dupe from %s: %s", znfp1(&msg->node_from), msgid);
 	    msgs_dupe++;
 	    if(!kill_dupe)
 		return do_bad_msg(msg, body);
@@ -786,8 +934,8 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	{
 	    /* Routed EchoMail */
 	    log("routed echomail area %s from %s to %s", area->area,
-		node_to_asc(&msg->node_from, TRUE),
-		node_to_asc(&msg->node_to, TRUE)                    );
+		znfp1(&msg->node_from),
+		znfp2(&msg->node_to)                    );
 	    msgs_routed++;
 	    if(!kill_routed)
 		return do_bad_msg(msg, body);
@@ -805,7 +953,7 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	{
 	    /* Insecure EchoMail */
 	    log("insecure echomail area %s from %s", area->area,
-		node_to_asc(&msg->node_from, TRUE)              );
+		znfp1(&msg->node_from)              );
 	    msgs_insecure++;
 	    if(!kill_insecure)
 		return do_bad_msg(msg, body);
@@ -834,7 +982,7 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 	lon_add(&seenby, &msg->node_from);
     if( !lon_search_echo(&seenby, &msg->node_to  ) )
 	lon_add(&seenby, &msg->node_to);
-
+    
     /* Add nodes not already in SEEN-BY to seenby and new */
     do_seenby(&seenby, &area->nodes, &new);
 
@@ -842,6 +990,22 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
     if(addto)
 	do_seenby(&seenby, &addto->add, NULL);
 
+    /* Add all AKAs for the current zone, if AddOtherAKA is set */
+    if(add_other_aka) 
+    {
+	z = cf_zone();
+	for(node=cf_addr_trav(FIRST); node; node=cf_addr_trav(NEXT))
+	    if(node->zone == z               &&
+	       !lon_search_echo(&seenby, node)  )
+		lon_add(&seenby, node);
+    }
+    
+#ifdef AI_3
+    /* Delete nodes from SEEN-BY */
+    if(deleteseenby_first)
+	do_deleteseenby(&seenby);
+#endif
+
     /* If not passthru area and not from our own address (point gateway
      * setup with Address==Uplink), add our own address to new          */
     if(! p_flag                               &&
@@ -852,6 +1016,12 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
     /* Add our address to end of ^APATH, if not already there */
     do_path(&path);
 
+#ifdef AI_3
+    /* Delete nodes from PATH */
+    if(deletepath_first)
+	do_deletepath(&path);
+#endif
+
     /* After */
     lon_debug(9, "SEEN-BY: ", &seenby, FALSE);
     lon_debug(9, "Path   : ", &path,   FALSE);
@@ -864,8 +1034,8 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
     {
 	    /* Circular ^APATH EchoMail */
 	    log("circular path echomail area %s from %s to %s",
-		area->area, node_to_asc(&msg->node_from, TRUE),
-		node_to_asc(&msg->node_to, TRUE)                 );
+		area->area, znfp1(&msg->node_from),
+		znfp2(&msg->node_to)                 );
 	    msgs_path++;
 
 	    lon_delete(&seenby);
@@ -908,8 +1078,8 @@ int do_echomail(Packet *pkt, Message *msg, MsgBody *body)
 void add_via(Textlist *list, Node *gate)
 {
     tl_appendf(list, "\001Via FIDOGATE/%s %s, %s\r\n",
-		     PROGRAM, node_to_asc(gate, FALSE),
-		     date("%a %b %d %Y at %H:%M:%S %Z", NULL)  );
+		     PROGRAM, znfp1(gate),
+		     date(DATE_VIA, NULL)  );
 }
 
 
@@ -946,7 +1116,7 @@ void do_rewrite(Message *msg)
 	    node = msg->node_from;
 	    change_addr(&msg->node_from, &r->to);
 	    log("rewrite: %s -> %s",
-		node_to_asc(&node, TRUE), node_to_asc(&msg->node_from, TRUE) );
+		znfp1(&node), znfp2(&msg->node_from) );
 	    break;
 	}
 
@@ -957,7 +1127,7 @@ void do_rewrite(Message *msg)
 	    node = msg->node_to;
 	    change_addr(&msg->node_to, &r->to);
 	    log("rewrite: %s -> %s",
-		node_to_asc(&node, TRUE), node_to_asc(&msg->node_to, TRUE) );
+		znfp1(&node), znfp2(&msg->node_to) );
 	    break;
 	}
 }
@@ -997,12 +1167,12 @@ int do_remap(Message *msg)
 	    {
 		if(r->type == CMD_REMAP_TO)
 		    log("remapto: %s @ %s -> %s", msg->name_to,
-			node_to_asc(&node, TRUE),
-			!kill ? node_to_asc(&msg->node_to, TRUE) : "KILLED");
+			znfp1(&node),
+			!kill ? znfp2(&msg->node_to) : "KILLED");
 		else
 		    log("remapfrom: %s @ %s -> %s", msg->name_from,
-			node_to_asc(&msg->node_from, TRUE),
-			!kill ? node_to_asc(&msg->node_to, TRUE) : "KILLED");
+			znfp1(&msg->node_from),
+			!kill ? znfp2(&msg->node_to) : "KILLED");
 	    }
 	    
 	    break;
@@ -1049,8 +1219,8 @@ int do_netmail(Packet *pkt, Message *msg, MsgBody *body)
 
     if(log_netmail)
 	log("MAIL: %s @ %s -> %s @ %s",
-	    msg->name_from, node_to_asc(&msg->node_from, TRUE),
-	    msg->name_to  , node_to_asc(&msg->node_to  , TRUE) );
+	    msg->name_from, znfp1(&msg->node_from),
+	    msg->name_to  , znfp2(&msg->node_to) );
 
     /*
      * Check for file attach
@@ -1068,7 +1238,7 @@ int do_netmail(Packet *pkt, Message *msg, MsgBody *body)
 	if( is_local_addr(&msg->node_to, FALSE) )
 	{
 	    log("killing empty msg from %s @ %s",
-		msg->name_from, node_to_asc(&msg->node_from, TRUE));
+		msg->name_from, znfp1(&msg->node_from));
 	    msgs_empty++;
 	    
 	    return OK;
@@ -1152,11 +1322,11 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	if(feof(pkt_file))
 	{
 	    log("WARNING: premature EOF reading input packet");
-	    return OK;
+	    TMPS_RETURN(OK);
 	}
 	
 	log("ERROR: reading input packet");
-	return ERROR;
+	TMPS_RETURN(ERROR);
     }
 
     while(type == MSG_TYPE)
@@ -1169,7 +1339,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	if(pkt_get_msg_hdr(pkt_file, &msg) == ERROR)
 	{
 	    log("ERROR: reading input packet");
-	    return ERROR;
+	    TMPS_RETURN(ERROR);
 	}
 	
 	/*
@@ -1185,7 +1355,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    else
 	    {
 		log("ERROR: reading input packet");
-		return ERROR;
+		TMPS_RETURN(ERROR);
 	    }
 	}
 	msgs_in++;
@@ -1199,15 +1369,17 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	/* Retrieve address information from kludges for NetMail */
 	if(body.area == NULL)
 	{
+	    /* Don't use point address from packet for Netmail */
+	    msg.node_from.point = 0;
+	    msg.node_to  .point = 0;
 	    /* Retrieve complete address from kludges */
 	    kludge_pt_intl(&body, &msg, TRUE);
 	    msg.node_orig = msg.node_from;
 
 	    debug(5, "NetMail: %s -> %s",
-		  node_to_asc(&msg.node_from, TRUE),
-		  node_to_asc(&msg.node_to  , TRUE) );
+		  znfp1(&msg.node_from), znfp2(&msg.node_to) );
 	    if(do_netmail(pkt, &msg, &body) == ERROR)
-		return ERROR;
+		TMPS_RETURN(ERROR);
 	}
 	else 
 	{
@@ -1217,10 +1389,10 @@ int unpack(FILE *pkt_file, Packet *pkt)
 		msg.node_orig = msg.node_from;
 
 	    debug(5, "EchoMail: %s -> %s",
-		  node_to_asc(&msg.node_from, TRUE),
-		  node_to_asc(&msg.node_to  , TRUE) );
+		  znfp1(&msg.node_from),
+		  znfp2(&msg.node_to) );
 	    if(do_echomail(pkt, &msg, &body) == ERROR)
-		return ERROR;
+		TMPS_RETURN(ERROR);
 	}
 
 	/*
@@ -1230,7 +1402,7 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	{
 	    outpkt_close();
 	    msg_count = 0;
-	    return severe_error=ERROR;
+	    TMPS_RETURN(severe_error=ERROR);
 	}
 	
 	/*
@@ -1245,9 +1417,10 @@ int unpack(FILE *pkt_file, Packet *pkt)
 	    msg_count = 0;
 	}
 
+	tmps_freeall();
     } /**while(type == MSG_TYPE)**/
 
-    return OK;
+    TMPS_RETURN(OK);
 }
 
 
@@ -1260,7 +1433,7 @@ int rename_bad(char *name)
     char bad[MAXPATH];
     int len;
     
-    strncpy0(bad, name, sizeof(bad));
+    BUF_COPY(bad, name);
     len = strlen(bad) - 4;
     if(len < 0)
 	len = 0;
@@ -1302,20 +1475,20 @@ int unpack_file(char *pkt_name)
     if(!pkt_file) {
 	log("$ERROR: can't open packet %s", pkt_name);
 	rename_bad(pkt_name);
-	return OK;
+	TMPS_RETURN(OK);
     }
     if(pkt_get_hdr(pkt_file, &pkt) == ERROR)
     {
 	log("ERROR: reading header from %s", pkt_name);
 	fclose(pkt_file);
 	rename_bad(pkt_name);
-	return OK;
+	TMPS_RETURN(OK);
     }
     
     /* Unpack it */
     pkt_size = check_size(pkt_name);
-    log("packet %s (%ldb) from %s for %s", pkt_name, pkt_size,
-	node_to_asc(&pkt.from, TRUE), node_to_asc(&pkt.to, TRUE) );
+    log("packet %s (%ldb) from %s to %s", pkt_name, pkt_size,
+	znfp1(&pkt.from), znfp2(&pkt.to) );
     pkts_in++;
     pkts_bytes += pkt_size;
     
@@ -1324,7 +1497,7 @@ int unpack_file(char *pkt_name)
 	log("ERROR: processing %s", pkt_name);
 	fclose(pkt_file);
 	rename_bad(pkt_name);
-	return severe_error;
+	TMPS_RETURN(severe_error);
     }
     
     fclose(pkt_file);
@@ -1332,10 +1505,10 @@ int unpack_file(char *pkt_name)
     if (unlink(pkt_name)) {
 	log("$ERROR: can't unlink %s", pkt_name);
 	rename_bad(pkt_name);
-	return ERROR;
+	TMPS_RETURN(ERROR);
     }
 
-    return OK;
+    TMPS_RETURN(OK);
 }
 
 
@@ -1383,9 +1556,10 @@ void usage(void)
     
     fprintf(stderr, "usage:   %s [-options] [packet ...]\n\n", PROGRAM);
     fprintf(stderr, "\
-options: -g --grade G                 processing grade\n\
-         -I --in-dir name             set input packet directory\n\
-         -O --out-dir name            set output packet directory\n\
+options: -d --no-dupecheck            disable dupe check\n\
+         -g --grade G                 processing grade\n\
+         -I --in-dir DIR              set input packet directory\n\
+         -O --out-dir DIR             set output packet directory\n\
          -l --lock-file               create lock file while processing\n\
          -t --insecure                insecure tossing (no AREAS.BBS check)\n\
          -n --toss-all                toss all EchoMail messages\n\
@@ -1399,9 +1573,7 @@ options: -g --grade G                 processing grade\n\
 \n\
 	 -v --verbose                 more verbose\n\
 	 -h --help                    this help\n\
-         -c --config name             read config file (\"\" = none)\n\
-	 -L --lib-dir name            set lib directory\n\
-	 -S --spool-dir name          set spool directory\n\
+         -c --config NAME             read config file (\"\" = none)\n\
 	 -a --addr Z:N/F.P            set FTN address\n\
 	 -u --uplink-addr Z:N/F.P     set FTN uplink address\n");
     
@@ -1418,8 +1590,8 @@ int main(int argc, char **argv)
     char *p;
     char *I_flag=NULL, *O_flag=NULL, *r_flag=NULL, *M_flag=NULL;
     char *c_flag=NULL;
-    char *S_flag=NULL, *L_flag=NULL;
     char *a_flag=NULL, *u_flag=NULL;
+    int d_flag = FALSE;
     char *pkt_name;
     char *areas_bbs = NULL;
     time_t toss_start, toss_delta;
@@ -1427,6 +1599,7 @@ int main(int argc, char **argv)
     int option_index;
     static struct option long_options[] =
     {
+	{ "no-dupecheck", 1, 0, 'd'},	/* Disable dupe check */
 	{ "grade",        1, 0, 'g'},	/* grade */
 	{ "in-dir",       1, 0, 'I'},	/* Set inbound packets directory */
 	{ "lock-file",    0, 0, 'l'},	/* Create lock file while processing */
@@ -1444,8 +1617,6 @@ int main(int argc, char **argv)
 	{ "verbose",      0, 0, 'v'},	/* More verbose */
 	{ "help",         0, 0, 'h'},	/* Help */
 	{ "config",       1, 0, 'c'},	/* Config file */
-	{ "spool-dir",    1, 0, 'S'},	/* Set FIDOGATE spool directory */
-	{ "lib-dir",      1, 0, 'L'},	/* Set FIDOGATE lib directory */
 	{ "addr",         1, 0, 'a'},	/* Set FIDO address */
 	{ "uplink-addr",  1, 0, 'u'},	/* Set FIDO uplink address */
 	{ 0,              0, 0, 0  }
@@ -1458,10 +1629,13 @@ int main(int argc, char **argv)
     cf_initialize();
 
     /* Parse options */
-    while ((c = getopt_long(argc, argv, "g:O:I:ltnr:sm:xM:b:pvhc:S:L:a:u:",
+    while ((c = getopt_long(argc, argv, "dg:O:I:ltnr:sm:xM:b:pvhc:a:u:",
 			    long_options, &option_index     )) != EOF)
 	switch (c) {
 	/***** ftntoss options *****/
+	case 'd':
+	    d_flag = TRUE;
+	    break;
 	case 'g':
 	    g_flag = *optarg;
 	    break;
@@ -1513,12 +1687,6 @@ int main(int argc, char **argv)
 	case 'c':
 	    c_flag = optarg;
 	    break;
-	case 'S':
-	    S_flag = optarg;
-	    break;
-	case 'L':
-	    L_flag = optarg;
-	    break;
 	case 'a':
 	    a_flag = optarg;
 	    break;
@@ -1534,17 +1702,11 @@ int main(int argc, char **argv)
     /*
      * Read config file
      */
-    if(L_flag)				/* Must set libdir beforehand */
-	cf_set_libdir(L_flag);
     cf_read_config_file(c_flag ? c_flag : CONFIG);
 
     /*
      * Process config options
      */
-    if(L_flag)
-	cf_set_libdir(L_flag);
-    if(S_flag)
-	cf_set_spooldir(S_flag);
     if(a_flag)
 	cf_set_addr(a_flag);
     if(u_flag)
@@ -1592,8 +1754,15 @@ int main(int argc, char **argv)
     }
     if(cf_get_string("DupeCheck", TRUE))
     {
-	debug(8, "config: DupeCheck");
-	dupe_check = TRUE;
+	if(d_flag) {
+	    debug(8, "config: DupeCheck disabled from command line!");
+	    dupe_check = FALSE;
+	}
+	else
+	{
+	    debug(8, "config: DupeCheck");
+	    dupe_check = TRUE;
+	}
     }
     if(cf_get_string("KillNoMSGID", TRUE))
     {
@@ -1629,9 +1798,9 @@ int main(int argc, char **argv)
 	    max_history = 0;
 	debug(8, "config: MaxHistory %lg", max_history);
     }
-    if( cf_get_string("EchoMail4D", TRUE) )
+    if( cf_get_string("TossEchoMail4D", TRUE) )
     {
-	debug(8, "config: EchoMail4D");
+	debug(8, "config: TossEchoMail4D");
 	echomail4d = TRUE;
     }
     if( cf_get_string("NoEmptyPath", TRUE) )
@@ -1639,27 +1808,26 @@ int main(int argc, char **argv)
 	debug(8, "config: NoEmptyPath");
 	no_empty_path = TRUE;
     }
+    if( cf_get_string("AddOtherAKA", TRUE) ||
+	cf_get_string("AddOtherAKAs", TRUE)  )
+    {
+	debug(8, "config: AddOtherAKA");
+	add_other_aka = TRUE;
+    }
 
     zonegate_init();
     addtoseenby_init();
+#ifdef AI_3
+    deleteseenby_init();
+    deletepath_init();
+#endif
 
     /*
      * Process local options
      */
-    if(I_flag)
-	str_expand_name(in_dir, sizeof(in_dir), I_flag);
-    else 
-	BUF_COPY3(in_dir, cf_spooldir(), "/", TOSS_IN);
-    if(O_flag)
-    {
-	pkt_outdir(O_flag, NULL);
-	pkt_baddir(O_flag, NULL);
-    }
-    else
-    {
-	pkt_outdir(cf_spooldir(), TOSS_TMP);
-	pkt_baddir(cf_spooldir(), TOSS_BAD);
-    }
+    BUF_EXPAND(in_dir, I_flag ? I_flag : cf_p_pinbound());
+    pkt_outdir(O_flag ? O_flag : DEFAULT_TOSS_TOSS, NULL);
+    pkt_baddir(O_flag ? O_flag : DEFAULT_TOSS_BAD , NULL);
     
     /*
      * Get name of areas.bbs file from config file
@@ -1675,7 +1843,7 @@ int main(int argc, char **argv)
 	exit(EX_USAGE);
     }
     
-    routing_init(r_flag ? r_flag : ROUTING);
+    routing_init(r_flag ? r_flag : cf_p_routing() );
     areasbbs_init(areas_bbs);
     passwd_init();
 
@@ -1710,7 +1878,7 @@ int main(int argc, char **argv)
 	/* Open history */
 	if(dupe_check)
 	{
-	    if(lock_program(LOCK_HISTORY, FALSE) == ERROR)
+	    if(lock_program(DEFAULT_LOCK_HISTORY, FALSE) == ERROR)
 	    {
 		/* Already busy, exit */
 		if(l_flag)
@@ -1737,7 +1905,7 @@ int main(int argc, char **argv)
 	/* Close history */
 	if(dupe_check)
 	{
-	    unlock_program(LOCK_HISTORY);
+	    unlock_program(DEFAULT_LOCK_HISTORY);
 	    hi_close();
 	}
 	
@@ -1758,7 +1926,7 @@ int main(int argc, char **argv)
 	/* Open history */
 	if(dupe_check)
 	{
-	    if(lock_program(LOCK_HISTORY, FALSE) == ERROR)
+	    if(lock_program(DEFAULT_LOCK_HISTORY, FALSE) == ERROR)
 	    {
 		/* Already busy, exit */
 		if(l_flag)
@@ -1786,7 +1954,7 @@ int main(int argc, char **argv)
 	/* Close history */
 	if(dupe_check)
 	{
-	    unlock_program(LOCK_HISTORY);
+	    unlock_program(DEFAULT_LOCK_HISTORY);
 	    hi_close();
 	}
 	
